@@ -33,6 +33,7 @@
 #define MYNAME "GARMIN" 
 static const char *portname;
 static short_handle mkshort_handle;
+static GPS_PWay *tx_waylist;
 static GPS_PWay *tx_routelist;
 static GPS_PWay *cur_tx_routelist_entry;
 static GPS_PTrack *tx_tracklist;
@@ -40,6 +41,7 @@ static GPS_PTrack *cur_tx_tracklist_entry;
 static int my_track_count = 0;
 static char *getposn = NULL;
 static char *poweroff = NULL;
+static char *eraset = NULL;                                                           
 static char *resettime = NULL;
 static char *snlen = NULL;
 static char *snwhiteopt = NULL;
@@ -67,6 +69,8 @@ arglist_t garmin_args[] = {
 		NULL, ARGTYPE_BOOL, ARG_NOMINMAX},
 	{ "power_off", &poweroff, "Command unit to power itself down", 
 		NULL, ARGTYPE_BOOL, ARG_NOMINMAX},
+       { "erase_t", &eraset, "Erase existing courses when writing new ones",          
+               NULL, ARGTYPE_BOOL, ARG_NOMINMAX},                                     
 	{ "resettime", &resettime, "Sync GPS time to computer time", 
 		NULL, ARGTYPE_BOOL, ARG_NOMINMAX},
 	{ "category", &category, "Category number to use for written waypoints", 
@@ -186,6 +190,7 @@ rw_init(const char *fname)
 				case 694: /* Legend HCx */
 				case 695: /* Vista HC */
 				case 786: /* HC model */
+				case 957: /* Legend HC */
 					receiver_short_length = 14;
 					snwhiteopt = xstrdup("1");
 					receiver_must_upper = 0;
@@ -193,6 +198,7 @@ rw_init(const char *fname)
 					receiver_charset = CET_CHARSET_MS_ANSI;
 					break;
 				case 291: /* GPSMAP 60CS, probably others */
+				case 1095: /* GPS 72H */
 					receiver_short_length = 10;
 					valid_waypt_chars = MILITANT_VALID_WAYPT_CHARS " +-";
 					setshort_badchars(mkshort_handle, "\"$.,'!");
@@ -207,6 +213,14 @@ rw_init(const char *fname)
                                         receiver_must_upper = 0;
                                         receiver_short_length = 14;
                                         break;
+				case 429: // Streetpilot i3
+					receiver_must_upper = 0;
+					receiver_charset = CET_CHARSET_MS_ANSI;
+					receiver_short_length = 30;
+					break;
+                                case 484: // Forerunner 305
+                                        receiver_short_length = 8;
+                                        break;
 				case 260: /* GPSMap 296 */
 				default:
 					break;
@@ -216,6 +230,13 @@ rw_init(const char *fname)
 			break;
 			
 	}
+
+	// If a user has specified a non-default character set, we'll trust
+	// them to sort our the wreckage of violating the Garmin protocol and
+	// ship characters to the device in that character set.
+	if (global_opts.charset != &cet_cs_vec_utf8) {
+		receiver_charset = global_opts.charset_name;
+	}
 	if (global_opts.debug_level > 0)  {
 		fprintf(stderr, "Waypoint type: %d\n"
 			"Chosen waypoint length %d\n",
@@ -224,8 +245,12 @@ rw_init(const char *fname)
 		fprintf(stderr, "Waypoint category type: %d\n",
 		gps_category_type);
 	}
-
 	}
+
+	// Allow override of sent character set for internationalized GPSes.
+	if (global_opts.charset != &cet_cs_vec_utf8)
+		receiver_charset = xstrdup(global_opts.charset_name);
+
 	/*
 	 * If the user provided a short_length, override the calculated value.
 	 */
@@ -400,11 +425,10 @@ track_read(void)
 	route_head *trk_head = NULL;
 	int trk_num = 0;
 	int i;
-	int trk_seg_num = 1;
-	char trk_seg_num_buf[10];
 	char *trk_name = "";
 	GPS_PLap* laps = NULL;
 	int nlaps = 0;
+	int next_is_new_trkseg = 0;
 
 	if (gps_lap_type != -1) {
 		nlaps = GPS_Command_Get_Lap(portname, &laps, &lap_read_nop_cb);
@@ -428,26 +452,22 @@ track_read(void)
 			trk_name = array[i]->trk_ident;
 			if (!trk_name)
 				trk_name = "";
-			trk_seg_num = 1;
 		}
 
-
-		if ((trk_head == NULL) || array[i]->tnew) {
+		if (trk_head == NULL || array[i]->ishdr) {
 			trk_head = route_head_alloc();
 			trk_head->rte_num = trk_num;
-			if (trk_seg_num == 1) {
-				trk_head->rte_name = xstrdup(trk_name);
-			} else {
-				/* name in the form TRACKNAME #n */
-				snprintf(trk_seg_num_buf, sizeof(trk_seg_num_buf), "%d", trk_seg_num);
-				trk_head->rte_name = xmalloc(strlen(trk_name)+strlen(trk_seg_num_buf)+3);
-				sprintf(trk_head->rte_name, "%s #%s", trk_name, trk_seg_num_buf);
-			}
-			trk_seg_num++;
+			trk_head->rte_name = xstrdup(trk_name);
 			trk_num++;
 			track_add_head(trk_head);
 		}
 
+		/* Need to do this here because fitness devices set tnew
+		 * on a trackpoint without lat/lon.
+		 */
+		if (array[i]->tnew)
+			next_is_new_trkseg = 1;
+
 		if (array[i]->no_latlon || array[i]->ishdr) {
 			continue;
 		}
@@ -462,7 +482,14 @@ track_read(void)
 		wpt->creation_time = array[i]->Time;
                 wpt->wpt_flags.is_split = checkWayPointIsAtSplit(wpt, laps,
                                                                  nlaps);
-		
+                wpt->wpt_flags.new_trkseg = next_is_new_trkseg;
+                next_is_new_trkseg = 0;
+
+		if (array[i]->dpth < 1.0e25f)
+			WAYPT_SET(wpt, depth, array[i]->dpth);
+		if (array[i]->temperature_populated)
+			WAYPT_SET(wpt, temperature, array[i]->temperature);
+
 		track_add_wpt(trk_head, wpt);
 	}
 
@@ -628,16 +655,13 @@ pvt2wpt(GPS_PPvt_Data pvt, waypoint *wpt)
         wpt->longitude = pvt->lon;
 	WAYPT_SET(wpt,course,1);
 	WAYPT_SET(wpt,speed,1);
-	/* convert to true course in degrees */
-	if ( pvt->east >= 0.0 )
-		wpt->course = 90 - DEG(atan(pvt->north/pvt->east));
-	else
-		wpt->course = 270 - DEG(atan(pvt->north/pvt->east));
-#if 0
+
+	wpt->course = 180 + DEG(atan2(-pvt->east, -pvt->north));
+
 	/* velocity in m/s */
-	wpt->speed = sqrt(pvt->north*pvt->north + pvt->east*pvt->east);
-	wpt->vs = pvt->up;
-#endif
+	WAYPT_SET(wpt,speed, sqrt(pvt->north*pvt->north + pvt->east*pvt->east));
+	// wpt->vs = pvt->up;
+
 	/*
 	 * The unit reports time in three fields:
 	 * 1) The # of days to most recent Sun. since  1989-12-31 midnight UTC.
@@ -762,7 +786,7 @@ sane_GPS_Way_New(void)
 	way->cross_road[0] = 0;
 	way->cross_road[0] = 0;
 	way->dpth = 1.0e25f;
-	way->wpt_class = 0;
+	way->wpt_class = 0;  // user waypoint by default.
 
 	return way;
 }
@@ -799,21 +823,19 @@ get_gc_info(waypoint *wpt)
 	return "";
 }
 
-static void
-waypoint_write(void)
+static int
+waypoint_prepare(void)
 {
 	int i;
-	int32 ret;
 	int n = waypt_count();
 	queue *elem, *tmp;
 	extern queue waypt_head;
-	GPS_PWay *way;
 	int icon;
 
-	way = xcalloc(n,sizeof(*way));
+	tx_waylist = xcalloc(n,sizeof(*tx_waylist));
 
 	for (i = 0; i < n; i++) {
-		way[i] = sane_GPS_Way_New();
+		tx_waylist[i] = sane_GPS_Way_New();
 	}
 
 	i = 0;
@@ -840,16 +862,18 @@ waypoint_write(void)
 		 * but rather a garmin "fixed length" buffer that's padded
 		 * to the end with spaces.  So this is NOT (strlen+1).
 		 */
-		memcpy(way[i]->ident, ident, strlen(ident));
+		memcpy(tx_waylist[i]->ident, ident, strlen(ident));
 
 		if (global_opts.synthesize_shortnames) { 
 			xfree(ident);
 		}
-		way[i]->ident[sizeof(way[i]->ident)-1] = 0;
+		tx_waylist[i]->ident[sizeof(tx_waylist[i]->ident)-1] = 0;
 
 		// If we were explictly given a comment from GPX, use that. 
-		if (wpt->description) {
-			memcpy(way[i]->cmnt, wpt->description, strlen(wpt->description));
+		//  This logic really is horrible and needs to be untangled.
+		if (wpt->description && 
+		    global_opts.smart_names && !wpt->gc_data->diff) {
+			memcpy(tx_waylist[i]->cmnt, wpt->description, strlen(wpt->description));
 		} else {
 			if (global_opts.smart_names && 
 			     wpt->gc_data->diff && wpt->gc_data->terr) {
@@ -860,16 +884,16 @@ xasprintf(&src, "%s %s", &wpt->shortname[2], src);
 						get_gc_info(wpt),
 						wpt->gc_data->diff, wpt->gc_data->terr, 
 						src);
-				memcpy(way[i]->cmnt, obuf, strlen(obuf));
+				memcpy(tx_waylist[i]->cmnt, obuf, strlen(obuf));
 			} else  {
-				memcpy(way[i]->cmnt, src, strlen(src));
+				memcpy(tx_waylist[i]->cmnt, src, strlen(src));
 			}
 		}
 
 		
 
-		way[i]->lon = wpt->longitude;
-		way[i]->lat = wpt->latitude;
+		tx_waylist[i]->lon = wpt->longitude;
+		tx_waylist[i]->lat = wpt->latitude;
 
 		if (deficon) {
 			icon = gt_find_icon_number_from_desc(deficon, PCX);
@@ -887,41 +911,52 @@ xasprintf(&src, "%s %s", &wpt->shortname[2], src);
 		if (gps_waypt_type == 103) {
 			icon = d103_icon_number_from_symbol(wpt->icon_descr);
 		}
-		way[i]->smbl = icon;
+		tx_waylist[i]->smbl = icon;
 		if (wpt->altitude == unknown_alt) {
-			way[i]->alt_is_unknown = 1;
-			way[i]->alt = 0;
+			tx_waylist[i]->alt_is_unknown = 1;
+			tx_waylist[i]->alt = 0;
 		} else {
-			way[i]->alt = wpt->altitude;
+			tx_waylist[i]->alt = wpt->altitude;
 		}
 		if (wpt->creation_time) {
-			way[i]->time = wpt->creation_time;
-			way[i]->time_populated = 1;
+			tx_waylist[i]->time = wpt->creation_time;
+			tx_waylist[i]->time_populated = 1;
 		}
 		if (category) {
-			way[i]->category = 1 << (atoi(category) - 1);
+			tx_waylist[i]->category = 1 << (atoi(category) - 1);
 		}
 		if (categorybits) {
-			way[i]->category = categorybits;
+			tx_waylist[i]->category = categorybits;
 		}
 #if SOON
-		garmin_fs_garmin_before_write(wpt, way[i], gps_waypt_type);
+		garmin_fs_garmin_before_write(wpt, tx_waylist[i], gps_waypt_type);
 #endif
 		i++;
 	}
 
-	if ((ret = GPS_Command_Send_Waypoint(portname, way, n, waypt_write_cb)) < 0) {
+	return n;
+}
+
+static void
+waypoint_write(void)
+{
+	int i, n;
+	int32 ret;
+
+	n = waypoint_prepare();
+
+	if ((ret = GPS_Command_Send_Waypoint(portname, tx_waylist, n, waypt_write_cb)) < 0) {
 		fatal(MYNAME ":communication error sending wayoints..\n");
 	}
 
 	for (i = 0; i < n; ++i) {
-		GPS_Way_Del(&way[i]);
+		GPS_Way_Del(&tx_waylist[i]);
 	}
 	if (global_opts.verbose_status) {
 		fprintf(stdout, "\r\n");
 		fflush(stdout);
 	}
-	xfree(way);
+	xfree(tx_waylist);
 }
 
 static void
@@ -957,6 +992,9 @@ route_waypt_pr(const waypoint *wpt)
 	rte->lat = wpt->latitude;
 	rte->smbl = gt_find_icon_number_from_desc(wpt->icon_descr, PCX);
 
+        // map class so unit doesn't duplicate routepoints as a waypoint.
+	rte->wpt_class = 0x80;  
+
 	if (wpt->altitude != unknown_alt) {
 		rte->alt = wpt->altitude;
 	} else {
@@ -1013,7 +1051,6 @@ route_write(void)
 static void
 track_hdr_pr(const route_head *trk_head)
 {
-	(*cur_tx_tracklist_entry)->tnew = gpsTrue;
 	(*cur_tx_tracklist_entry)->ishdr = gpsTrue;
 	if ( trk_head->rte_name ) {
 		strncpy((*cur_tx_tracklist_entry)->trk_ident, trk_head->rte_name, sizeof((*cur_tx_tracklist_entry)->trk_ident));
@@ -1030,20 +1067,21 @@ track_waypt_pr(const waypoint *wpt)
 {
 	(*cur_tx_tracklist_entry)->lat = wpt->latitude;
 	(*cur_tx_tracklist_entry)->lon = wpt->longitude;
-	(*cur_tx_tracklist_entry)->alt = wpt->altitude;
+	(*cur_tx_tracklist_entry)->alt = (wpt->altitude != unknown_alt) ? wpt->altitude : 1e25;
 	(*cur_tx_tracklist_entry)->Time = wpt->creation_time;
 	if ( wpt->shortname ) {
 		strncpy((*cur_tx_tracklist_entry)->trk_ident, wpt->shortname, sizeof((*cur_tx_tracklist_entry)->trk_ident));
 		(*cur_tx_tracklist_entry)->trk_ident[sizeof((*cur_tx_tracklist_entry)->trk_ident)-1] = 0;
 	}
+	(*cur_tx_tracklist_entry)->tnew = wpt->wpt_flags.new_trkseg;
 	cur_tx_tracklist_entry++;
 }
 
-static void
-track_write(void)
+static int
+track_prepare(void)
 {
 	int i;
-	int n = track_waypt_count() + track_count();
+	int32 n = track_waypt_count() + track_count();
 
 	tx_tracklist = xcalloc(n, sizeof(GPS_PTrack));
 	cur_tx_tracklist_entry = tx_tracklist;
@@ -1053,14 +1091,47 @@ track_write(void)
 	my_track_count = 0;
 	track_disp_all(track_hdr_pr, route_noop, track_waypt_pr);
 
-	GPS_Command_Send_Track(portname, tx_tracklist, n);
+	GPS_Prepare_Track_For_Device(&tx_tracklist, &n);
 
+	return n;
+}
+
+static void
+track_write(void)
+{
+	int i, n;
+
+	n = track_prepare();
+       GPS_Command_Send_Track(portname, tx_tracklist, n, (eraset)? 1 : 0);            
+                                                                                      
 	for (i = 0; i < n; i++) {
 		GPS_Track_Del(&tx_tracklist[i]);
 	}
 	xfree(tx_tracklist);
 }
 
+static void
+course_write(void)
+{
+	int i, n_trk, n_wpt;
+
+	n_wpt = waypoint_prepare();
+	n_trk = track_prepare();
+
+	GPS_Command_Send_Track_As_Course(portname, tx_tracklist, n_trk,
+                                        tx_waylist, n_wpt, (eraset)? 1 : 0);          
+
+	for (i = 0; i < n_wpt; ++i) {
+		GPS_Way_Del(&tx_waylist[i]);
+	}
+	xfree(tx_waylist);
+
+	for (i = 0; i < n_trk; i++) {
+		GPS_Track_Del(&tx_tracklist[i]);
+	}
+	xfree(tx_tracklist);
+}
+
 static void
 data_write(void)
 {
@@ -1068,12 +1139,25 @@ data_write(void)
 		return;
 	}
 
-	if (global_opts.masked_objective & WPTDATAMASK)
-	  waypoint_write();
+	/* If we have both trackpoints and waypoints and the device
+	 * supports courses, combine them to a course. Otherwise,
+	 * send tracks & waypoints separately.
+	 */
+	if ((global_opts.masked_objective & WPTDATAMASK) &&
+	    (global_opts.masked_objective & TRKDATAMASK) &&
+	    gps_course_transfer != -1)
+	{
+	  course_write();
+	}
+	else
+	{
+	    if (global_opts.masked_objective & WPTDATAMASK)
+		waypoint_write();
+	    if (global_opts.masked_objective & TRKDATAMASK)
+		track_write();
+	}
 	if (global_opts.masked_objective & RTEDATAMASK)
 	  route_write();
-	if (global_opts.masked_objective & TRKDATAMASK)
-	  track_write();
 }
 
 
