@@ -359,7 +359,7 @@ geoniche_read_asc(void)
     } 
 }
 
-static char *geoniche_icon_map[] = 			/* MPS */
+static const char *geoniche_icon_map[] = 			/* MPS */
 {
 	 /* 21 */ "Cross",
 	 /* 22 */ "Cross (light)",			
@@ -413,10 +413,10 @@ static char *geoniche_icon_map[] = 			/* MPS */
 	 /* 52 */ "Mystery or puzzle Cache",
 };
 
-static char *
+static const char *
 geoniche_icon_to_descr(const int no)
 {
-    char *result = NULL;
+    const char *result = NULL;
 
     if (no >= 0x21) 
     {
@@ -589,7 +589,7 @@ wpt2icon(const waypoint *wpt)
     else if (strstr(desc, "unk")) return 48;
     else if (strstr(desc, "cam")) return 49;
 
-    switch (wpt->gc_data.type) {
+    switch (wpt->gc_data->type) {
 	case gt_traditional: return 43;
 	case gt_multi: return 44;
 	case gt_locationless: return 45;
@@ -604,6 +604,7 @@ wpt2icon(const waypoint *wpt)
 	case gt_benchmark: return 0;
 	case gt_ape: return 0;
 	case gt_mega: return 0;
+	case gt_wherigo: return 0;
     }
 
     return 0;
@@ -615,11 +616,11 @@ geoniche_geostuff(const waypoint *wpt)
 	char *gs = NULL, *tmp1, *tmp2, *tmp3;
 	char tbuf[10240];
 
-	if (!wpt->gc_data.terr) {
+	if (!wpt->gc_data->terr) {
 		return NULL;
 	}
 
-	snprintf(tbuf, sizeof(tbuf), "\n%s by %s\n\n", gs_get_cachetype(wpt->gc_data.type), wpt->gc_data.placer);
+	snprintf(tbuf, sizeof(tbuf), "\n%s by %s\n\n", gs_get_cachetype(wpt->gc_data->type), wpt->gc_data->placer);
 	gs = xstrappend(gs, tbuf);
 
 /*
@@ -632,15 +633,15 @@ geoniche_geostuff(const waypoint *wpt)
 /*
  * 3 May 06: Added container type
  */
-	snprintf(tbuf, sizeof(tbuf), "Container: %s\nDifficulty: %3.1f\nTerrain: %3.1f\n\n", gs_get_container(wpt->gc_data.container), wpt->gc_data.diff/10.0, wpt->gc_data.terr/10.0);
+	snprintf(tbuf, sizeof(tbuf), "Container: %s\nDifficulty: %3.1f\nTerrain: %3.1f\n\n", gs_get_container(wpt->gc_data->container), wpt->gc_data->diff/10.0, wpt->gc_data->terr/10.0);
 	gs = xstrappend(gs, tbuf);
 
-	tmp1 = strip_html(&wpt->gc_data.desc_short);
-	tmp2 = strip_html(&wpt->gc_data.desc_long);
+	tmp1 = strip_html(&wpt->gc_data->desc_short);
+	tmp2 = strip_html(&wpt->gc_data->desc_long);
 	gs = xstrappend(gs, tmp1);
 	gs = xstrappend(gs, tmp2);
 
-	tmp3 = rot13(wpt->gc_data.hint);
+	tmp3 = rot13(wpt->gc_data->hint);
 	snprintf(tbuf, sizeof(tbuf), "\n\nHint: %s\n", tmp3);
 	gs = xstrappend(gs, tbuf);
 
