@@ -21,9 +21,10 @@ int32  GPS_A200_Send(const char *port, GPS_PWay *way, int32 n);
 int32  GPS_A201_Send(const char *port, GPS_PWay *way, int32 n);
 
 int32  GPS_A300_Get(const char *port, GPS_PTrack **trk, pcb_fn cb);
-int32  GPS_A301_Get(const char *port, GPS_PTrack **trk, pcb_fn cb);
+int32  GPS_A301_Get(const char *port, GPS_PTrack **trk, pcb_fn cb, int protoid);
 int32  GPS_A300_Send(const char *port, GPS_PTrack *trk, int32 n);
-int32  GPS_A301_Send(const char *port, GPS_PTrack *trk, int32 n); /*A302*/
+int32  GPS_A301_Send(const char *port, GPS_PTrack *trk, int32 n, int protoid,
+                    gpsdevh *fd);
 
 int32  GPS_D300_Get(GPS_PTrack *trk, int32 entries, gpsdevh *h);
 void   GPS_D300b_Get(GPS_PTrack *trk, UC *data);
@@ -32,10 +33,11 @@ void   GPS_D302b_Get(GPS_PTrack *trk, UC *data);
 void   GPS_D303b_Get(GPS_PTrack *trk, UC *data); /*D304*/
 void   GPS_D310_Get(GPS_PTrack *trk, UC *s);
 void   GPS_D311_Get(GPS_PTrack *trk, UC *s);
-void   GPS_D300_Send(UC *data, GPS_PTrack trk);
-void   GPS_D301_Send(UC *data, GPS_PTrack trk);
-void   GPS_D304_Send(UC *data, GPS_PTrack trk);
+void   GPS_D300_Send(UC *data, GPS_PTrack trk, int32 *len);
+void   GPS_D301_Send(UC *data, GPS_PTrack trk, int32 *len, int type);
+void   GPS_D303_Send(UC *data, GPS_PTrack trk, int32 *len, int protoid);
 void   GPS_D310_Send(UC *data, GPS_PTrack trk, int32 *len);
+void   GPS_D311_Send(UC *data, GPS_PTrack trk, int32 *len);
 
 int32  GPS_A400_Get(const char *port, GPS_PWay **way);
 int32  GPS_A400_Send(const char *port, GPS_PWay *way, int32 n);
@@ -58,8 +60,29 @@ int32  GPS_A800_Off(const char *port, gpsdevh **fd);
 int32  GPS_A800_Get(gpsdevh **fd, GPS_PPvt_Data *packet);
 void   GPS_D800_Get(GPS_PPacket packet, GPS_PPvt_Data *pvt);
 
-int32 GPS_A906_Get(const char *port, GPS_PLap **lap, pcb_fn cb);
-void GPS_D1011b_Get(GPS_PLap *Lap,UC *data); /*D906 D1001 D1015*/
+int32  GPS_A906_Get(const char *port, GPS_PLap **lap, pcb_fn cb);
+void   GPS_D1011b_Get(GPS_PLap *Lap,UC *data); /*D906 D1001 D1015*/
+
+int32  GPS_A1006_Get(const char *port, GPS_PCourse **crs, pcb_fn cb);
+int32  GPS_A1006_Send(const char *port, GPS_PCourse *crs, int32 n_crs,
+                      gpsdevh *fd);
+void   GPS_D1006_Get(GPS_PCourse *crs, UC *p);
+void   GPS_D1006_Send(UC *data, GPS_PCourse crs, int32 *len);
+
+int32  GPS_A1007_Get(const char *port, GPS_PCourse_Lap **clp, pcb_fn cb);
+int32  GPS_A1007_Send(const char *port, GPS_PCourse_Lap *clp, int32 n_clp,
+                      gpsdevh *fd);
+void   GPS_D1007_Get(GPS_PCourse_Lap *clp, UC *p);
+void   GPS_D1007_Send(UC *data, GPS_PCourse_Lap clp, int32 *len);
+
+int32  GPS_A1008_Get(const char *port, GPS_PCourse_Point **cpt, pcb_fn cb);
+int32  GPS_A1008_Send(const char *port, GPS_PCourse_Point *cpt, int32 n_cpt,
+                      gpsdevh *fd);
+void   GPS_D1012_Get(GPS_PCourse_Point *cpt, UC *p);
+void   GPS_D1012_Send(UC *data, GPS_PCourse_Point cpt, int32 *len);
+
+int32  GPS_A1009_Get(const char *port, GPS_PCourse_Limits limits);
+void   GPS_D1013_Get(GPS_PCourse_Limits limits, UC *p);
 
 /* Unhandled documented protocols, as of:
   Garmin Device Interface Specification, May 19, 2006, Drawing Number: 001-00063-00 Rev. C
@@ -75,12 +98,6 @@ A1004  Fitness User Profile Transfer Protocol
 	Capability A1004: D1004
 A1005  Workout Limits Transfer Protocol
 	Capability A1005: D1005
-A1006  Course Transfer Protocol
-	Capability A1006: D1006
-	Capability A1007: D1007
-	Capability A1008: D1012
-A1009  Course Limits Transfer Protocol
-	Capability A1009: D1013
 */
 /* Unimplemted and Undocumented, as listed from the following device/sw:
 	GF305 3.70
@@ -95,8 +112,9 @@ Capability A918: D918
 Capability A1013: D1014
 */
 
-const char * Get_Pkt_Type(unsigned char p, unsigned short d0, const char **xinfo);
+const char * Get_Pkt_Type(US p, US d0, const char **xinfo);
 
+void GPS_Prepare_Track_For_Device(GPS_PTrack **trk, int32 *n);
 
 #endif
 
