@@ -20,11 +20,15 @@
  */
 
 
+#if HAVE_CONFIG_H
 #include "config.h"
+#endif
+
 #include "../defs.h"
+
 #if !HAVE_LIBUSB 
 
-const char no_usb[] = "USB suport is not available in this build.\n";
+const char no_usb[] = "USB support is not available in this build.\n";
 
 int
 gusb_init(const char *portname)
