@@ -1,7 +1,7 @@
 /*
     Magellan ".gs" files as they appear on USB of Explorist 400,500,600.
 
-    Copyright (C) 2005, 2006 robertlipe@usa.net
+    Copyright (C) 2005, 2006, 2008 robertlipe@usa.net
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -21,18 +21,21 @@
 
 #include <ctype.h>
 #include "defs.h"
+#include "csv_util.h"
 #include "xmlgeneric.h"
 #include "magellan.h"
 
 #define MYNAME "maggeo"
 
 /* Turn this on (remove) after 5.2 becomes widespread. */
-#define FIRMWARE_DOES_88591 0		
+#define FIRMWARE_DOES_88591 0
 
 static gbfile *maggeofile_in;
 static gbfile *maggeofile_out;
 static short_handle desc_handle = NULL;
 
+static time_t maggeo_parsedate(char *dmy);
+
 static void
 maggeo_writemsg(const char * const buf)
 {
@@ -75,7 +78,82 @@ maggeo_wr_deinit(void)
 static void
 maggeo_read(void)
 {
-	fatal(MYNAME ": Reading maggeo is not implemented yet.\n");
+	char *buff;
+
+	while ((buff = gbfgetstr(maggeofile_in))) {
+		waypoint *wpt_tmp;
+        	geocache_data *gcdata;
+		char *s = NULL;
+		int fld;
+
+		buff = lrtrim(buff);
+		if (*buff == '\0') continue;
+		if (strncmp(buff, "$PMGNGEO,", 9)) continue;
+
+		buff += 9; /* skip field no. 1 */
+		fld = 1;
+
+		wpt_tmp = waypt_new();
+		gcdata = waypt_alloc_gc_data(wpt_tmp);
+
+		while ((s = csv_lineparse(buff, ",", "", fld++))) {
+		  buff = NULL;
+
+		  s = lrtrim(s);
+		  if (*s == '\0') continue;
+
+                  switch(fld) {
+                    case 2:
+                              wpt_tmp->latitude = ddmm2degrees(atof(s));
+                              break;
+                    case 3:
+                              if (s[0] == 'S')
+                                wpt_tmp->latitude = -wpt_tmp->latitude;
+                              break;
+                    case 4:
+                              wpt_tmp->longitude = ddmm2degrees(atof(s));
+                              break;
+                    case 5:
+                              if (s[0] == 'W')
+                                wpt_tmp->longitude = -wpt_tmp->longitude;
+                              break;
+                    case 6:
+                              wpt_tmp->altitude = atof(s);
+                              break;
+                    case 7:
+                              if (s[0] == 'F') wpt_tmp->altitude = METERS_TO_FEET(wpt_tmp->altitude);
+                              break;
+                    case 8:
+                              wpt_tmp->shortname = xstrdup(s);
+                              break;
+                    case 9:
+                              wpt_tmp->description = xstrdup(s);
+                              break;
+                    case 10:
+                              gcdata->placer = xstrdup(s);
+                              break;
+                    case 11:
+                              gcdata->hint = xstrdup(s);
+                              break;
+                    case 12: // cache type
+                              gcdata->type = gs_mktype(s);
+                              break;
+                    case 13:  
+		    	      wpt_tmp->creation_time = maggeo_parsedate(s);
+                              break;
+                    case 14: // last found date is ignored.
+                              break;
+                    case 15:
+                              gcdata->diff = 10 * atof(s);
+                              break;
+                    case 16:
+                              gcdata->terr = 10 * atof(s);
+                              break;
+                  }
+                }
+                waypt_add(wpt_tmp);
+	}
+
 }
 
 /*
@@ -87,21 +165,53 @@ maggeo_fmtdate(time_t t)
 {
 	#define SZ 16
 
-	char *cbuf = xmalloc(SZ);
 	struct tm *tm = NULL;
 	int date;
-	tm = gmtime(&t);
+	char *cbuf = xmalloc(SZ);
 
-	if ( t && tm ) {
-		date = tm->tm_mday * 100000 + (1+tm->tm_mon) * 1000 + 
-			   tm->tm_year;
-		snprintf(cbuf, SZ, "%07d", date);
-	} else {
-		cbuf[0] = '\0';
+	cbuf[0] = '\0';
+	if (t > 0) {
+		tm = gmtime(&t);
+		if ( tm ) {
+			date = tm->tm_mday * 100000 + (1+tm->tm_mon) * 1000 + 
+				   tm->tm_year;
+			snprintf(cbuf, SZ, "%07d", date);
+		}
 	}
 	return cbuf;
 }
 
+/*
+ * The maggeo date format s DDMMYYY where "YYY" is the number
+ * of years since 1900.  This, of course, means anything in this
+ * century is three digits but anything from before 2000, we'd have
+ * two digit years.  This makes this easier to parse as strings.
+ */
+static time_t maggeo_parsedate(char *dmy)
+{
+        struct tm tm;
+        char dd[3];
+        char mm[3];
+
+	if (strlen(dmy) < 5) return 0;
+
+        memset(&tm, 0, sizeof(tm));
+
+        dd[0] = dmy[0];
+        dd[1] = dmy[1];
+        dd[2] = 0;
+
+        mm[0] = dmy[2];
+        mm[1] = dmy[3];
+        mm[2] = 0;
+
+        tm.tm_mday = atoi(dd);
+        tm.tm_mon = atoi(mm) - 1;
+        tm.tm_year = atoi(dmy + 4);
+
+        return mktime(&tm);
+}
+
 /*
  * Append an optional UTF string to buf, prepending a comma, 
  * cleansing it of NMEA-isms and decomposing to ASCII as we go.
@@ -167,15 +277,15 @@ maggeo_waypt_pr(const waypoint *waypointp)
 	 * For some reason, Magellan used exactly the GPX spellings of 
 	 * everything except this one...
 	 */
-	if (waypointp->gc_data.type == gt_suprise) {
+	if (waypointp->gc_data->type == gt_suprise) {
 		ctype = "Mystery Cache";
 	} else {
-		ctype = gs_get_cachetype(waypointp->gc_data.type);
+		ctype = gs_get_cachetype(waypointp->gc_data->type);
 	}
 	placeddate = maggeo_fmtdate(waypointp->creation_time);
-	lfounddate = maggeo_fmtdate(waypointp->gc_data.last_found);
+	lfounddate = maggeo_fmtdate(waypointp->gc_data->last_found);
 	cname = mkshort(desc_handle, waypointp->notes ? waypointp->notes : waypointp->description);
-	placer = waypointp->gc_data.placer;
+	placer = waypointp->gc_data->placer;
 
 	/*
 	 * As of this writing on 05/04, the firmware in the units will
@@ -198,20 +308,20 @@ maggeo_waypt_pr(const waypoint *waypointp)
 	append(obuf, shortname);
 	append(obuf, cname);
 	append(obuf, placer);
-	append(obuf, waypointp->gc_data.hint);
+	append(obuf, waypointp->gc_data->hint);
 	append(obuf, ctype);
 	append(obuf, placeddate);
 	append(obuf, lfounddate);
 
-	if (waypointp->gc_data.diff/10.0)
+	if (waypointp->gc_data->diff/10.0)
 		sprintf(obuf + strlen(obuf), ",%3.1f", 
-			waypointp->gc_data.diff/10.0);
+			waypointp->gc_data->diff/10.0);
 	else
 		strcat(obuf, ",");
 
-	if (waypointp->gc_data.terr/10.0)
+	if (waypointp->gc_data->terr/10.0)
 		sprintf(obuf + strlen(obuf), ",%3.1f", 
-			waypointp->gc_data.terr/10.0);
+			waypointp->gc_data->terr/10.0);
 	else
 		strcat(obuf, ",");
 
@@ -230,7 +340,7 @@ maggeo_write(void)
 
 ff_vecs_t maggeo_vecs = {
 	ff_type_file,
-	{ ff_cap_write, ff_cap_none, ff_cap_none },
+	{ ff_cap_read | ff_cap_write, ff_cap_none, ff_cap_none },
 	maggeo_rd_init,
 	maggeo_wr_init,
 	maggeo_rd_deinit,
