@@ -26,6 +26,15 @@
 	2008/10/18: Initial release
 	2008/10/19: Don't write empty names
 		    Add options 'locked' and 'visible'
+	2008/11/06: Fix enumeration of objects for empty routes or tracks
+		    Add option "ver" (internal version) to writer
+		    We support write of version:
+		    * 0x11 - reported as " "Memory Map OS Edition 2004, Version 4.2.3 Build 432"
+		    * 0x12 - most files in my test pool :-)
+	2008/11/19: Fix routes with a loop but different start and end point
+	2008/12/12: Fix object release error
+	2010/09/10: Added read support for version 0x18
+		    (test file created by Memory-Map European Edition, Version 5.4.2, Build 1089).
 */
 
 #include <ctype.h>
@@ -43,7 +52,7 @@
 
 // #define MMO_DBG
 
-static char *opt_locked, *opt_visible;
+static char *opt_locked, *opt_visible, *opt_version;
 
 static
 arglist_t mmo_args[] = {
@@ -51,6 +60,8 @@ arglist_t mmo_args[] = {
 		ARGTYPE_BOOL, ARG_NOMINMAX },
 	{ "visible", &opt_visible, "Write items 'visible' [default yes]", "1",
 		ARGTYPE_BOOL, ARG_NOMINMAX },
+	{ "ver", &opt_version, "Write files with internal version [n]", NULL,
+		ARGTYPE_INT, "17", "18" },
 	ARG_TERMINATOR
 };
 
@@ -62,11 +73,12 @@ typedef struct mmo_data_s {
 	time_t ctime;
 	time_t mtime;
 	int left;		/* number of un-readed route points */
-	int done;		/* number of completely loaded route points */
 	void *data;		/* can be a waypoint, a route or a track */
+	int refct;
+	struct mmo_data_s **members;
 	unsigned char visible:1;
 	unsigned char locked:1;
-	unsigned char loop:1;	/* loop flag */
+	unsigned char loaded:1;
 } mmo_data_t;
 
 static gbfile *fin, *fout;
@@ -123,6 +135,12 @@ static const mmo_icon_mapping_t mmo_icon_value_table[] = {
 	{ -1, NULL }
 };
 
+static const gbuint32 obj_type_ico = 0x00;
+static const gbuint32 obj_type_rte = 0x14;
+static const gbuint32 obj_type_trk = 0x1E;
+static const gbuint32 obj_type_txt = 0x32;
+static const gbuint32 obj_type_wpt = 0x3C;
+
 /* helpers */
 
 #ifdef MMO_DBG
@@ -139,7 +157,7 @@ dbgprintf(const char *sobj, const char *fmt, ...)
 
 # define DBG(args) dbgprintf args
 #else
-# define DBG(args) ;
+# define DBG(args) do {} while (0) ;
 #endif
 
 static char *
@@ -240,12 +258,28 @@ mmo_get_object(const gbuint16 objid)
 	mmo_data_t *data;
 	
 	snprintf(key, sizeof(key), "%d", objid | 0x8000);
-	if (! avltree_find(objects, key, (void *)&data))
+	if (! avltree_find(objects, key, (void *)&data)) {
+#ifdef MMO_DBG
+		gbfseek(fin, -2, SEEK_CUR);
+		int ni, n;
+		for (ni = 0; (n = gbfgetc(fin)) != EOF; ni++) {
+			DBG(("mmo_get_object", "%04X %02X %c (%d)\n",
+				ni, n, n >= 32 && n <= 126 ? (char)n : '.', n));
+		}
+#endif
 		fatal(MYNAME ": Unregistered object id 0x%04X!\n", objid | 0x8000);
+	}
 	
 	return data;
 }
 
+static waypoint *
+mmo_get_waypt(mmo_data_t *data)
+{
+	data->refct++;
+	if (data->refct == 1) return (waypoint *)data->data;
+	else return waypt_dupe((waypoint *)data->data);
+}
 
 static void
 mmo_release_avltree(avltree_t *tree, const int is_object)
@@ -259,6 +293,8 @@ mmo_release_avltree(avltree_t *tree, const int is_object)
 			if (is_object) {
 				mmo_data_t *data = (mmo_data_t *)name;
 				if (data->name) xfree(data->name);
+				if ((data->type == wptdata) && (data->refct == 0))
+					waypt_free((waypoint *)data->data);
 			}
 			xfree(name);
 		} while ((key = avltree_next(tree, key, (void *)&name)));
@@ -277,7 +313,7 @@ mmo_register_icon(const int id, const char *name)
 }
 
 
-static mmo_data_t *mmo_read_object(const waypoint *add);
+static mmo_data_t *mmo_read_object(void);
 
 
 static void
@@ -287,15 +323,7 @@ mmo_end_of_route(mmo_data_t *data)
 	const char *sobj = "CObjRoute";
 #endif
 	route_head *rte = data->data;
-	int rtept = rte->rte_waypt_ct;
-	int i;
-	char buf[64];
-		
-	if (data->visible && data->loop) {
-		DBG((sobj, "route \"%s\" is a loop.\n", data->name));
-		(void) mmo_read_object(NULL);
-		rtept--;
-	}
+	char buf[7];
 
 	if (mmo_version >= 0x12) {
 		mmo_fillbuf(buf, 7, 1);
@@ -306,17 +334,7 @@ mmo_end_of_route(mmo_data_t *data)
 		rte->line_color.opacity = 255 - (buf[6] * 51);
 		DBG((sobj, "color = 0x%06X\n", rte->line_color.bbggrr));
 		DBG((sobj, "transparency = %d (-> %d)\n", buf[6], rte->line_color.opacity));
-	}
-
-	if (data->visible) {
-		for (i = 0; i < rtept; i++) (void) mmo_read_object(NULL);
-	}
-	if (data->loop && (data->done > 1)) {
-		queue *elem;
-					
-		elem = QUEUE_FIRST(&rte->waypoint_list);
-		dequeue(elem);
-		ENQUEUE_TAIL(&rte->waypoint_list, elem);
+		DBG((sobj, "for \"%s\" \n", data->name));
 	}
 
 	if (rte->rte_waypt_ct == 0) {	/* don't keep empty routes */
@@ -334,8 +352,9 @@ mmo_read_category(mmo_data_t *data)
 	if (marker & 0x8000) {
 		mmo_data_t *tmp;
 		
+		DBG(("mmo_read_category", "reading category object\n"));
 		gbfseek(fin, -2, SEEK_CUR);
-		tmp = mmo_read_object(NULL);
+		tmp = mmo_read_object();
 		if (data) data->category = tmp->name;
 	}
 }
@@ -347,22 +366,41 @@ mmo_read_CObjIcons(mmo_data_t *data)
 #ifdef MMO_DBG
 	const char *sobj = "CObjIcons";
 #endif
-	int i;
+	int icon_id;
 
 	DBG((sobj, ":-----------------------------------------------------\n"));
 	DBG((sobj, "name = \"%s\" [ visible=%s, id=0x%04X ]\n", 
 		data->name, data->visible ? "yes" : "NO", data->objid));
 
-	gbfseek(fin, 6, SEEK_CUR);	/* skip 6 unknown bytes */
+	if (mmo_version >= 0x18) {
+		gbuint16 u16;
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+	}
+	gbuint16 u16;
+	u16 = gbfgetuint16(fin);
+	DBG((sobj, "unknown value = 0x%04X\n", u16));
+	u16 = gbfgetuint16(fin);
+	DBG((sobj, "unknown value = 0x%04X\n", u16));
+	u16 = gbfgetuint16(fin);
+	DBG((sobj, "unknown value = 0x%04X\n", u16));
 
-	while ((i = gbfgetuint32(fin))) {
+	while ((icon_id = gbfgetuint32(fin))) {
 		char *name;
 		(void) gbfgetuint32(fin);
 		(void) gbfgetuint32(fin);
 		name = mmo_readstr();
-//		DBG((sobj, "bitmap(%d) = \"%s\"\n", i, name));
-		mmo_register_icon(i, name);
+		DBG((sobj, "bitmap(0x%08X) = \"%s\"\n", icon_id, name));
+		mmo_register_icon(icon_id, name);
 		xfree(name);
+		// The next four bytes hold the length of the image,
+		// read them and then skip the image data.
 		gbfseek(fin, gbfgetuint32(fin), SEEK_CUR);
 	}
 }
@@ -386,13 +424,25 @@ mmo_read_CObjWaypoint(mmo_data_t *data)
 	DBG((sobj, "name = \"%s\" [ visible=%s, id=0x%04X ]\n", 
 		data->name, data->visible ? "yes" : "NO", data->objid));
 
-	wpt = waypt_new();
+	data->data = wpt = waypt_new();
 	wpt->shortname = xstrdup(data->name);
 
 	time = data->mtime;
 	if (! time) time = data->ctime;
 	if (time > 0) wpt->creation_time = time;
 
+	if (mmo_version >= 0x18) {
+		gbuint16 u16;
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+	}
+
 	wpt->latitude = gbfgetdbl(fin);
 	wpt->longitude = gbfgetdbl(fin);
 
@@ -406,23 +456,9 @@ mmo_read_CObjWaypoint(mmo_data_t *data)
 
 		for (i = 0; i < rtelinks; i++) {
 			mmo_data_t *tmp;
-			int objid;			
 
 			DBG((sobj, "read rtelink number %d\n", i + 1));
-
-			objid = gbfgetuint16(fin);
-			gbfseek(fin, -2, SEEK_CUR);
-
-			rtelink[i] = tmp = mmo_read_object(wpt);
-			
-			if ((objid < 0x8000) && (tmp != NULL) && (tmp->type == rtedata)) {
-				route_head *rte = tmp->data;
-
-				tmp->left--;
-				route_add_wpt(rte, waypt_dupe(wpt));
-
-				DBG((sobj, "\"%s\" Added to route \"%s\"\n", wpt->shortname, rte->rte_name));
-			}
+			rtelink[i] = tmp = mmo_read_object();
 		}
 
 	}
@@ -461,6 +497,11 @@ mmo_read_CObjWaypoint(mmo_data_t *data)
 			wpt->wpt_flags.icon_descr_is_dynamic = 1;
 			DBG((sobj, "icon = \"%s\"\n", wpt->icon_descr));
 		}
+#ifdef MMO_DBG
+		else {
+			DBG((sobj, "icon not found for 0x%08X\n", i));
+		}
+#endif
 	}
 
 	wpt->proximity = le_read_float(&buf[4]);
@@ -479,38 +520,16 @@ mmo_read_CObjWaypoint(mmo_data_t *data)
 
 	ux = gbfgetuint32(fin);
 	DBG((sobj, "proximity type = %d\n", ux));
+	
+	data->loaded = 1;
 
-	if (rtelinks) {
-		int i;
-
-		for (i = 0; i < rtelinks; i++) {
-			int j;
-			route_head *rte = rtelink[i]->data;
-
-			for (j = 0; j < rtelinks; j++) {
-				if ((i != j) && (rtelink[i] == rtelink[j])) {
-					rtelink[i]->loop = 1;
-					break;
-				}
-			}
-			rtelink[i]->done++;
-			if ((rtelink[i]->left == 0) && (rtelink[i]->done == rte->rte_waypt_ct)) {
-				if (mmo_version <= 0x11) mmo_end_of_route(rtelink[i]);
-			}
-		}
-	}
-
-	if (rtelink) {
-		xfree(rtelink);
-		waypt_free(wpt);
-		data->data = NULL;
-	}
-	else waypt_add(wpt);
+	if (rtelink) xfree(rtelink);
+	else waypt_add(mmo_get_waypt(data));
 }
 
 
 static void
-mmo_read_CObjRoute(mmo_data_t *data, const waypoint *add)
+mmo_read_CObjRoute(mmo_data_t *data)
 {
 #ifdef MMO_DBG
 	const char *sobj = "CObjRoute";
@@ -528,26 +547,56 @@ mmo_read_CObjRoute(mmo_data_t *data, const waypoint *add)
 	rte->rte_name = xstrdup(data->name);
 	route_add_head(rte);
 
+	if (mmo_version >= 0x18) {
+		gbuint16 u16;
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+	}
+
 	ux = gbfgetc(fin);		/* line label */
 	DBG((sobj, "line label = %d\n", ux));
 
 	data->left = rtept = gbfgetint16(fin);
 	DBG((sobj, "route has %d point(s)\n", rtept));
 
-	if ((data->left <= 0) && (mmo_version >= 0x12)) {
-		mmo_fillbuf(buf, 7, 1);
+	if (data->left <= 0) {
+		if (mmo_version >= 0x12) mmo_fillbuf(buf, 7, 1);
+		route_del_head(rte);
+		data->data = NULL;
+
+		return;
 	}
 
-	if (add) {	/* waypoint loaded before route */
-		route_add_wpt(rte, waypt_dupe(add));
-		data->left--;
-	}
-	
 	while (data->left > 0) {
-		(void) mmo_read_object(NULL);
+		mmo_data_t *tmp;
+		
+		DBG((sobj, "read next waypoint\n"));
+		tmp = mmo_read_object();
+		if (tmp && tmp->data && (tmp->type = wptdata)) {
+			waypoint *wpt;
+			
+			/* FIXME: At this point this waypoint maybe not fully loaded (initialized) !!!
+				  We need a final procedure to handle this !!! */
+			if (! tmp->loaded) {
+				wpt = waypt_new();
+				wpt->latitude = 0;
+				wpt->longitude = 0;
+				xasprintf(&wpt->shortname, "\01%p", tmp);
+			}
+			else wpt = mmo_get_waypt(tmp);
+			
+			route_add_wpt(rte, wpt);
+			data->left--;
+		}
 	}
-	
-	if ((mmo_version > 0x11) && (data->done > 0)) mmo_end_of_route(data);
+
+	if (mmo_version > 0x11) mmo_end_of_route(data);
 }
 
 
@@ -568,6 +617,18 @@ mmo_read_CObjTrack(mmo_data_t *data)
 	trk->rte_name = xstrdup(data->name);
 	track_add_head(trk);
 
+	if (mmo_version >= 0x18) {
+		gbuint16 u16;
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+		u16 = gbfgetuint16(fin);
+		DBG((sobj, "unknown value = 0x%04X (since 0x18)\n", u16));
+	}
+
 	tp = gbfgetint16(fin);
 	DBG((sobj, "track has %d point(s)\n", tp));
 
@@ -579,7 +640,9 @@ mmo_read_CObjTrack(mmo_data_t *data)
 		
 		wpt->latitude = gbfgetdbl(fin);
 		wpt->longitude = gbfgetdbl(fin);
+		DBG((sobj, "coordinates = %f / %f\n", wpt->latitude, wpt->longitude));
 		unk = gbfgetc(fin);
+		DBG((sobj, "Unknown = 0x%02X (%d)\n", unk, unk));
 		
 		wpt->creation_time = gbfgetint32(fin);
 		wpt->altitude = gbfgetflt(fin);
@@ -587,11 +650,11 @@ mmo_read_CObjTrack(mmo_data_t *data)
 		if (unk != 0) {
 			gbuint16 ux;
 			ux = gbfgetuint16(fin);
-			DBG((sobj, "u16 = %04X (%d)\n", ux, ux));
+			DBG((sobj, "unknown value = 0x%04X (%d)\n", ux, ux));
 			if (unk > 1) {
 				gbuint16 ux;
 				ux = gbfgetuint16(fin);
-				DBG((sobj, "u16 = %04X (%d)\n", ux, ux));
+				DBG((sobj, "unknown value = 0x%04X (%d)\n", ux, ux));
 			}
 		}
 		track_add_wpt(trk, wpt);
@@ -603,15 +666,15 @@ mmo_read_CObjTrack(mmo_data_t *data)
 		u32 = gbfgetuint32(fin); 	/* Min. update interval */
 		DBG((sobj, "min. update interval = %d\n", u32));
 		u32 = gbfgetuint32(fin); 	/* unknown */
-//		DBG((sobj, "unknown value = 0x%8X (%d)\n", u32, u32));
+		DBG((sobj, "unknown value = 0x%08X (%d)\n", u32, u32));
 		u32 = gbfgetuint32(fin); 	/* unknown */
-//		DBG((sobj, "unknown value = 0x%8X (%d)\n", u32, u32));
+		DBG((sobj, "unknown value = 0x%08X (%d)\n", u32, u32));
 		u32 = gbfgetuint32(fin); 	/* unknown */
 		DBG((sobj, "min. update distance = %d\n", u32));
 		u32 = gbfgetuint32(fin); 	/* unknown */
 		DBG((sobj, "track partition interval = %d\n", u32 / 60));
 		u32 = gbfgetuint32(fin); 	/* unknown */
-//		DBG((sobj, "unknown value = 0x%8X (%d)\n", u32, u32));
+		DBG((sobj, "unknown value = 0x%08X (%d)\n", u32, u32));
 		u32 = gbfgetuint32(fin); 	/* unknown */
 		DBG((sobj, "tick interval = %d\n", u32 / 60));
 		trk->line_color.bbggrr = gbfgetuint32(fin); 	/* rgb color */
@@ -635,11 +698,11 @@ mmo_read_CObjTrack(mmo_data_t *data)
 			gbuint16 u16;
 			
 			u8 = gbfgetc(fin);
-//			DBG((sobj, "u8 = 0x%X (since 0x16)\n", u8));
+			DBG((sobj, "unknown value = 0x%02X (since 0x16)\n", u8));
 			u16 = gbfgetuint16(fin);
-//			DBG((sobj, "u16 = 0x%X (since 0x16)\n", u16));
+			DBG((sobj, "unknown value = 0x%04X (since 0x16)\n", u16));
 			u16 = gbfgetuint16(fin);
-//			DBG((sobj, "u16 = 0x%X (since 0x16)\n", u16));
+			DBG((sobj, "unknown value = 0x%04X (since 0x16)\n", u16));
 		}
 	}
 	
@@ -656,8 +719,7 @@ mmo_read_CObjText(mmo_data_t *data)
 #ifdef MMO_DBG
 	const char *sobj = "CObjText";
 #endif
-	int i;
-	char buf[512];
+	char buf[28];
 	double lat, lon;
 	char *text, *font;
 			
@@ -672,15 +734,14 @@ mmo_read_CObjText(mmo_data_t *data)
 	text = mmo_readstr();
 	DBG((sobj, "text = \"%s\"\n", text));
 	xfree(text);
-	
+
 	mmo_fillbuf(buf, 28, 1);
 
 	font = mmo_readstr();
 	DBG((sobj, "font = \"%s\"\n", font));
 	xfree(font);
 
-	i = mmo_fillbuf(buf, 25, 1);
-//	mmo_printbuf(buf, i, "CObjText\n");
+	mmo_fillbuf(buf, 25, 1);
 }
 
 
@@ -715,13 +776,19 @@ mmo_read_CObjCurrentPosition(mmo_data_t *data)
 
 
 static mmo_data_t *
-mmo_read_object(const waypoint *add)
+mmo_read_object(void)
 {
 	int objid;
 	mmo_data_t *data = NULL;
 
+	// There are three cases:
+	// a new object of a type that has not occurred previously in this file;
+	// a new object; or
+	// a back reference to an object that appears earlier in the file.
+
 	objid = gbfgetuint16(fin);
 	if (objid == 0xFFFF) {
+		DBG(("mmo_read_object", "Registering new object type\n"));
 		gbuint16 version;
 		char *sobj;
 		int len;
@@ -736,6 +803,7 @@ mmo_read_object(const waypoint *add)
 		sobj = xmalloc(len + 1);
 		sobj[len] = '\0';
 		gbfread(sobj, len, 1, fin);
+		DBG(("mmo_read_object", "%s\n", sobj));
 		
 		if (strcmp(sobj, "CObjIcons") == 0) ico_object_id = objid;
 		else if (strcmp(sobj, "CCategory") == 0) cat_object_id = objid;
@@ -749,8 +817,9 @@ mmo_read_object(const waypoint *add)
 		xfree(sobj);
 	}
 
-	if (objid & 0x8000) {
+	DBG(("mmo_read_object", "objid = 0x%04X\n", objid));
 
+	if (objid & 0x8000) {
 		data = mmo_register_object(mmo_object_id++, NULL, 0);
 		data->name = mmo_readstr();
 
@@ -759,10 +828,25 @@ mmo_read_object(const waypoint *add)
 			data->mtime = gbfgetuint32(fin);
 			data->locked = gbfgetc(fin);
 			data->visible = gbfgetc(fin);
-			(void) gbfgetuint16(fin);
-			(void) gbfgetuint16(fin);
-		
+
+			gbuint32 obj_type;
+			obj_type = gbfgetuint32(fin);
+#ifdef MMO_DBG
+			gbuint32 expected_type = 0xFFFFFFFF;
+			if      (objid == ico_object_id) expected_type = obj_type_ico;
+			else if (objid == trk_object_id) expected_type = obj_type_trk;
+			else if (objid == wpt_object_id) expected_type = obj_type_wpt;
+			else if (objid == rte_object_id) expected_type = obj_type_rte;
+			else if (objid == txt_object_id) expected_type = obj_type_txt;
+			if (mmo_version >= 0x18) expected_type <<= 24;
+			DBG(("mmo_read_object", "object type = 0x%08X\n", obj_type));
+			if (obj_type != expected_type)
+				DBG(("mmo_read_object", "   expected   0x%08X\n", expected_type));
+#endif
+
 			if (objid != ico_object_id) mmo_read_category(data);
+			DBG(("mmo_read_object", "Category : %s\n",
+				data->category ? data->category : "[No category]"));
 		}
 
 		if (objid == cat_object_id) ; 	/* do nothing */		
@@ -777,7 +861,7 @@ mmo_read_object(const waypoint *add)
 		}
 		else if (objid == rte_object_id) {
 			data->type = rtedata;
-			mmo_read_CObjRoute(data, add);
+			mmo_read_CObjRoute(data);
 		}
 		else if (objid == pos_object_id) mmo_read_CObjCurrentPosition(data);
 		else if (objid == txt_object_id) mmo_read_CObjText(data);
@@ -789,6 +873,38 @@ mmo_read_object(const waypoint *add)
 	return data;
 }
 
+static void
+mmo_finalize_rtept_cb(const waypoint *wptref)
+{
+	waypoint *wpt = (waypoint *)wptref;
+	
+	if ((wpt->shortname[0] == 1) && (wpt->latitude == 0) && (wpt->longitude == 0)) {
+		mmo_data_t *data;
+		waypoint *wpt2;
+
+		sscanf(wpt->shortname + 1, "%p", &data);
+		wpt2 = (waypoint *)data->data;
+
+		wpt->latitude = wpt2->latitude;
+		wpt->longitude = wpt2->longitude;
+
+		xfree(wpt->shortname);
+		wpt->shortname = xstrdup(wpt2->shortname);
+
+		if (wpt2->description) wpt->description = xstrdup(wpt2->description);
+		if (wpt2->notes) wpt->notes = xstrdup(wpt2->notes);
+		if (wpt2->url) wpt->notes = xstrdup(wpt2->url);
+
+		wpt->proximity = wpt2->proximity;
+		wpt->wpt_flags.proximity = wpt2->wpt_flags.proximity;
+
+		if (wpt2->icon_descr) {
+			wpt->icon_descr = xstrdup(wpt2->icon_descr);
+			wpt->wpt_flags.icon_descr_is_dynamic = 1;
+		}
+	}
+}
+
 /*******************************************************************************
 * %%%        global callbacks called by gpsbabel main process              %%% *
 *******************************************************************************/
@@ -820,6 +936,8 @@ mmo_rd_init(const char *fname)
 static void 
 mmo_rd_deinit(void)
 {
+	route_disp_session(curr_session(), NULL, NULL, mmo_finalize_rtept_cb);
+
 	mmo_release_avltree(icons, 0);
 	mmo_release_avltree(category_ids, 0);
 	mmo_release_avltree(objects, 1);
@@ -850,7 +968,7 @@ mmo_read(void)
 	DBG((sobj, "number of objects = %d\n", mmo_obj_ct));
 	
 	i = gbfgetuint16(fin);
-	if (i != 0xFFFF) fatal(MYNAME ": Marker not equel to 0xFFFF!\n");
+	if (i != 0xFFFF) fatal(MYNAME ": Marker not equal to 0xFFFF!\n");
 
 	mmo_version = gbfgetuint16(fin);
 	DBG((sobj, "version = 0x%02X\n", mmo_version));
@@ -862,7 +980,7 @@ mmo_read(void)
 
 	while (! gbfeof(fin)) {		/* main read loop */
 
-		(void) mmo_read_object(NULL);
+		(void) mmo_read_object();
 
 	}
 
@@ -910,7 +1028,7 @@ mmo_enum_waypt_cb(const waypoint *wpt)
 static void
 mmo_enum_route_cb(const route_head *rte)
 {
-	mmo_obj_ct++;
+	if (rte->rte_waypt_ct > 0) mmo_obj_ct++;
 }
 
 
@@ -930,7 +1048,8 @@ mmo_write_obj_mark(const char *sobj, const char *name)
 		mmo_object_id++;
 		snprintf(buf, sizeof(buf), "%u", mmo_object_id);
 
-		DBG(("write", "object \"%s\" registered (id = 0x%04X)\n", mmo_object_id));
+		DBG(("write", "object \"%s\", registered type \"%s\" (id = 0x%04X)\n",
+			name, sobj, mmo_object_id));
 
 		avltree_insert(mmobjects, sobj, xstrdup(buf));
 		
@@ -966,7 +1085,7 @@ mmo_write_category(const char *sobj, const char *name)
 
 static int
 mmo_write_obj_head(const char *sobj, const char *name, const time_t ctime, 
-	const gpsdata_type type)
+	const gbuint32 obj_type)
 {
 	int res;
 
@@ -978,13 +1097,7 @@ mmo_write_obj_head(const char *sobj, const char *name, const time_t ctime,
 	gbfputc(*opt_locked, fout);
 	gbfputc(*opt_visible, fout);
 
-	switch(type) {
-		case trkdata: gbfputuint16(0x1E, fout); break;
-		case wptdata: gbfputuint16(0x3C, fout); break;
-		case rtedata: gbfputuint16(0x14, fout); break;
-		default: gbfputuint16(type, fout); break;
-	}
-	gbfputuint16(0x00, fout);
+	gbfputuint32(obj_type, fout);
 	
 	return res;
 }
@@ -997,6 +1110,7 @@ mmo_write_wpt_cb(const waypoint *wpt)
 	int objid;
 	time_t time;
 	int icon = 0;
+	mmo_data_t *data;
 
 	time = wpt->creation_time;
 	if (time < 0) time = 0;
@@ -1014,9 +1128,12 @@ mmo_write_wpt_cb(const waypoint *wpt)
 		return;
 	}
 
+	DBG(("write", "waypoint \"%s\"\n", wpt->shortname ? wpt->shortname : "Mark"));
+
 	objid = mmo_write_obj_head("CObjWaypoint",
-		(wpt->shortname && *wpt->shortname) ? wpt->shortname : "Mark", time, wptdata);
-	mmo_register_object(objid, wpt, wptdata);
+		(wpt->shortname && *wpt->shortname) ? wpt->shortname : "Mark", time, obj_type_wpt);
+	data = mmo_register_object(objid, wpt, wptdata);
+	data->refct = 1;
 	mmo_write_category("CCategory", (mmo_datatype == rtedata) ? "Waypoints" : "Marks");
 
 	gbfputdbl(wpt->latitude, fout);
@@ -1097,7 +1214,7 @@ mmo_write_rte_head_cb(const route_head *rte)
 	if (time == 0x7FFFFFFF) time = gpsbabel_time;
 	
 	objid = mmo_write_obj_head("CObjRoute",
-		(rte->rte_name && *rte->rte_name) ? rte->rte_name : "Route", time, rtedata);
+		(rte->rte_name && *rte->rte_name) ? rte->rte_name : "Route", time, obj_type_rte);
 	mmo_register_object(objid, rte, rtedata);
 	mmo_write_category("CCategory", "Route");
 	gbfputc(0, fout); /* unknown */
@@ -1112,19 +1229,23 @@ mmo_write_rte_tail_cb(const route_head *rte)
 	
 	if (rte->rte_waypt_ct <= 0) return;
 
-	if (rte->line_color.bbggrr < 0) {
-		gbfputuint32(0xFF, fout);	/* color; default red */
-		gbfputc(0x01, fout);		/* Line width "normal" */
-		gbfputc(0x00, fout);		/* Line style "solid"*/
-		gbfputc(0x00, fout);		/* Transparency "Opaque" */
+	DBG(("write", "route with %d point(s).\n", rte->rte_waypt_ct));
+
+	if (mmo_version >= 0x12) {
+		if (rte->line_color.bbggrr < 0) {
+			gbfputuint32(0xFF, fout);	/* color; default red */
+			gbfputc(0x01, fout);		/* Line width "normal" */
+			gbfputc(0x00, fout);		/* Line style "solid"*/
+			gbfputc(0x00, fout);		/* Transparency "Opaque" */
+		}
+		else {
+			gbfputuint32(rte->line_color.bbggrr, fout);	/* color */
+			gbfputc(0x01, fout);		/* Line width "normal" */
+			gbfputc(0x00, fout);		/* Line style "solid"*/
+			gbfputc((255 - rte->line_color.opacity) / 51, fout);	/* Transparency "Opaque" */
+		}
 	}
-	else {
-		gbfputuint32(rte->line_color.bbggrr, fout);	/* color */
-		gbfputc(0x01, fout);		/* Line width "normal" */
-		gbfputc(0x00, fout);		/* Line style "solid"*/
-		gbfputc((255 - rte->line_color.opacity) / 51, fout);	/* Transparency "Opaque" */
-	}
-	
+
 	QUEUE_FOR_EACH(&rte->waypoint_list, elem, tmp) {
 		waypoint *wpt = (waypoint *)elem;
 		int objid = mmo_get_objid(wpt);
@@ -1141,7 +1262,7 @@ mmo_write_trk_head_cb(const route_head *trk)
 	if (trk->rte_waypt_ct <= 0) return;
 	
 	objid = mmo_write_obj_head("CObjTrack",
-		(trk->rte_name && *trk->rte_name) ? trk->rte_name : "Track", gpsbabel_time, trkdata);
+		(trk->rte_name && *trk->rte_name) ? trk->rte_name : "Track", gpsbabel_time, obj_type_trk);
 	mmo_write_category("CCategory", "Track");
 	gbfputuint16(trk->rte_waypt_ct, fout);
 	
@@ -1164,15 +1285,19 @@ mmo_write_trk_tail_cb(const route_head *trk)
 
 	if (trk->line_color.bbggrr < 0) {
 		gbfputuint32(0xFF0000, fout);	/* color; default blue */
-		gbfputc(0x01, fout);		/* Line width "normal" */
-		gbfputc(0x00, fout);		/* Line style "solid"*/
-		gbfputc(0x00, fout);		/* Transparency "Opaque" */
+		if (mmo_version >= 0x12) {
+			gbfputc(0x01, fout);		/* Line width "normal" */
+			gbfputc(0x00, fout);		/* Line style "solid"*/
+			gbfputc(0x00, fout);		/* Transparency "Opaque" */
+		}
 	}
 	else {
 		gbfputuint32(trk->line_color.bbggrr, fout);	/* color */
-		gbfputc(0x01, fout);		/* Line width "normal" */
-		gbfputc(0x00, fout);		/* Line style "solid"*/
-		gbfputc((255 - trk->line_color.opacity) / 51, fout);	/* Transparency "Opaque" */
+		if (mmo_version >= 0x12) {
+			gbfputc(0x01, fout);		/* Line width "normal" */
+			gbfputc(0x00, fout);		/* Line style "solid"*/
+			gbfputc((255 - trk->line_color.opacity) / 51, fout);	/* Transparency "Opaque" */
+		}
 	}
 }
 
@@ -1189,7 +1314,16 @@ mmo_wr_init(const char *fname)
 
 	mmo_object_id = 0x8000;
 	mmo_obj_ct = 1;			/* ObjIcons always present */
-	mmo_version = 0x12;		/* we write as version 0x12 */
+	mmo_version = 0x12;		/* by default we write as version 0x12 */
+	if (opt_version) {
+		while (isspace(*opt_version)) opt_version++;
+		errno = 0;
+		mmo_version = strtol(opt_version, NULL, 0);
+		if (errno || ((mmo_version != 0x11) && (mmo_version != 0x12))) {
+			fatal(MYNAME ": Unsupported version identifier (%s)!\n", opt_version);
+		}
+	}
+	DBG(("write", "version = 0x%02X\n", mmo_version));
 	mmo_filemark = 0xFFFFUL | (mmo_version << 16);
 }
 
@@ -1217,7 +1351,7 @@ mmo_write(void)
 	
 	gbfputuint16(mmo_obj_ct, fout);
 	
-	mmo_write_obj_head("CObjIcons", "Unnamed object", gpsbabel_time, 0);
+	mmo_write_obj_head("CObjIcons", "Unnamed object", gpsbabel_time, obj_type_ico);
 	for (i = 0; i < 5; i++) gbfputuint16(0, fout);
 
 	mmo_datatype = wptdata;
