@@ -34,8 +34,11 @@
  For info about the used log format:
   <http://spreadsheets.google.com/pub?key=pyCLH-0TdNe-5N-5tBokuOA&gid=5>
 
- Example usage::
-   
+  Module updated 2008-2009. Now also handles Holux M-241 and 
+  Holux GR-245 aka. GPSport-245 devices. These devices have small differences
+  in the log format and use a lower baudrate to transfer the data.
+
+ Example usage::  
    # Read from USB port, output trackpoints & waypoints in GPX format   
   ./gpsbabel -D 2 -t -w -i mtk -f /dev/ttyUSB0 -o gpx -F out.gpx
   
@@ -43,6 +46,7 @@
    #  both CSV file and GPX, discard points without fix
   ./gpsbabel -D 2 -t -i mtk-bin,csv=data__2007_09_04.csv -f data_2007_09_04.bin -x discard,fixnone -o gpx -F out.gpx
 
+  Tip: Check out the -x height,wgs84tomsl filter to correct the altitude. 
   Todo:
     o ....    
  
@@ -52,6 +56,7 @@
 #include "gbser.h"
 #include "gbfile.h" /* used for csv output */
 #include <ctype.h>
+#include <errno.h>
 
 #define MYNAME "mtk_logger"
 
@@ -108,6 +113,7 @@ static const char *MTK_ACK[] = { /* Flags returned from PMTK001 ack packet */
 #define MTK_EVT_DISTANCE (1<<0x04)
 #define MTK_EVT_SPEED    (1<<0x05)
 #define MTK_EVT_START    (1<<0x07)
+#define MTK_EVT_WAYPT    (1<<0x10)  /* Holux waypoint follows... */
 
 /* *************************************** */
 
@@ -139,7 +145,7 @@ struct log_type {
    int id;
    int size;
    const char *name;
-} log_type[] =  {
+} log_type[32] =  {
    { 0, 4, "UTC" },
    { 1, 2, "VALID" },
    { 2, 8, "LATITUDE,N/S"},
@@ -159,7 +165,8 @@ struct log_type {
    { 16, 2, "SNR"},
    { 17, 2, "RCR"},
    { 18, 2, "MILLISECOND"},
-   { 19, 8, "DISTANCE" }
+   { 19, 8, "DISTANCE" }, 
+   { 20, 0, NULL}, 
 };
 
 struct sat_info {
@@ -194,21 +201,33 @@ struct mtk_loginfo {
 
 /* *************************************** */
 
+/* MTK chip based devices with different baudrate, tweaks, ... */ 
+enum MTK_DEVICE_TYPE {
+    MTK_LOGGER, 
+    HOLUX_M241, 
+    HOLUX_GR245
+};
+
 #define TIMEOUT        1500
 #define MTK_BAUDRATE 115200
 #define MTK_BAUDRATE_M241 38400
 
+#define HOLUX245_MASK (1 << 27)
+
 static void *fd;  /* serial fd */
 static FILE *fl;  /* bin.file fd */
 static char *port; /* serial port name */
-static char *erase;  /* erase ? command option */
+static char *OPT_erase;  /* erase ? command option */
+static char *OPT_erase_only;  /* erase_only ? command option */
+static char *OPT_log_enable;  /* enable ? command option */
 static char *csv_file; /* csv ? command option */
-static int is_m241=0;
+static enum MTK_DEVICE_TYPE mtk_device = MTK_LOGGER;
 
 struct mtk_loginfo mtk_info;
 
 const char LIVE_CHAR[4] = {'-', '\\','|','/'};
-const char TEMP_DATA_BIN[]= "data.bin";
+static const char TEMP_DATA_BIN[]= "data.bin";
+static const char TEMP_DATA_BIN_OLD[]= "data_old.bin";
 
 
 const char CMD_LOG_DISABLE[]= "$PMTK182,5*20\r\n";
@@ -221,14 +240,19 @@ static int  mtk_log_len(unsigned int bitmask);
 static void mtk_rd_init(const char *fname);
 static void file_init(const char *fname);
 static void file_deinit(void) ;
+static void holux245_init(void);
 static void file_read(void);
 static int mtk_parse_info(const unsigned char *data, int dataLen);
 
 
-// Arguments for logg fetch 'mtk' command..
+// Arguments for log fetch 'mtk' command..
 
 static arglist_t mtk_sargs[] = {
-    { "erase", &erase, "Erase device data after download",
+    { "erase", &OPT_erase, "Erase device data after download",
+        "0", ARGTYPE_BOOL, ARG_NOMINMAX },
+    { "erase_only", &OPT_erase_only, "Only erase device data, do not download anything",
+	    "0", ARGTYPE_BOOL, ARG_NOMINMAX }, 
+    { "log_enable", &OPT_log_enable, "Enable logging after download",
         "0", ARGTYPE_BOOL, ARG_NOMINMAX },
     { "csv",   &csv_file, "MTK compatible CSV output file",
         NULL, ARGTYPE_STRING, ARG_NOMINMAX },
@@ -276,8 +300,8 @@ static int do_cmd(const char *cmd, const char *expect, char **rslt, time_t timeo
    cmd_erase = 0;
    if  ( strncmp(cmd, CMD_LOG_ERASE, 12) == 0 ){
       cmd_erase = 1;
-      if (global_opts.debug_level > 0 ) {
-         dbg(1, "Erasing    ");
+      if (global_opts.verbose_status || global_opts.debug_level > 0) {
+         fprintf(stderr, "Erasing    ");
       }
    }
    // dbg(6, "## Send '%s' -- Expect '%s' in %d sec\n", cmd, expect, timeout_sec);
@@ -302,13 +326,13 @@ static int do_cmd(const char *cmd, const char *expect, char **rslt, time_t timeo
     }
     loops++;
     dbg(8, "Read %d bytes: '%s'\n", len, line);
-    if ( cmd_erase && global_opts.debug_level > 0 && global_opts.debug_level <= 3 ){ 
+    if ( cmd_erase && (global_opts.verbose_status || (global_opts.debug_level > 0 && global_opts.debug_level <= 3)) ){ 
        // erase cmd progress wheel -- only for debug level 1-3 
        fprintf(stderr,"\b%c", LIVE_CHAR[loops%4]); fflush(stderr);
     }
       if ( len > 5 && line[0] == '$' ){
          if ( expect_len > 0 && strncmp(&line[1], expect, expect_len) == 0 ){
-            if ( cmd_erase && global_opts.debug_level > 0 ) fprintf(stderr,"\n");
+            if ( cmd_erase && (global_opts.verbose_status || global_opts.debug_level > 0) ) fprintf(stderr,"\n");
             dbg(6, "NMEA command success !\n");
             if ( (len - 4) > expect_len ){ // alloc and copy data segment...
                if ( line[len-3] == '*' ) 
@@ -352,42 +376,64 @@ static int do_cmd(const char *cmd, const char *expect, char **rslt, time_t timeo
 * %%%        global callbacks called by gpsbabel main process              %%% *
 *******************************************************************************/
 static void mtk_rd_init_m241 (const char *fname) {
-    is_m241 = 1;
+    mtk_device = HOLUX_M241;
     mtk_rd_init(fname);
 }
 
 static void mtk_rd_init(const char *fname){
     int rc;
+    char *model;
     
     port = xstrdup(fname);
-
+    
+    errno = 0;
     dbg(1, "Opening port %s...\n", fname);
-    if (fd = gbser_init(port), NULL == fd) {
-        fatal(MYNAME ": Can't initialise port \"%s\"\n", port);
+    if ( (fd = gbser_init(port)) == NULL ) {
+        fatal(MYNAME ": Can't initialise port \"%s\" (%s)\n", port, strerror(errno));
     }
     
     // verify that we have a MTK based logger...
     dbg(1, "Verifying MTK based device...\n");
 
-    if ( is_m241 ) {
-       log_type[LATITUDE].size = log_type[LONGITUDE].size = 4;
-       log_type[HEIGHT].size = 3;
-       rc = gbser_set_port(fd, MTK_BAUDRATE_M241, 8, 0, 1);
-    } else {
-       rc = gbser_set_port(fd, MTK_BAUDRATE, 8, 0, 1);
-    }
+    switch ( mtk_device ){
+       case HOLUX_M241:
+       case HOLUX_GR245:    
+          log_type[LATITUDE].size = log_type[LONGITUDE].size = 4;
+          log_type[HEIGHT].size = 3;
+          rc = gbser_set_port(fd, MTK_BAUDRATE_M241, 8, 0, 1);
+          break;
+       case MTK_LOGGER:
+       default:    
+          rc = gbser_set_port(fd, MTK_BAUDRATE, 8, 0, 1);
+          break;
+    }      
     if (rc) {
         dbg(1, "Set baud rate to %d failed (%d)\n", MTK_BAUDRATE, rc);
         fatal(MYNAME ": Failed to set baudrate !\n");
     }
 
-    rc = do_cmd("$PMTK605*31\r\n", "PMTK705", NULL, 10);
+    rc = do_cmd("$PMTK605*31\r\n", "PMTK705,", &model, 10);
     if ( rc != 0 )
-      fatal(MYNAME ": This is not a MTK based GPS ! (or is it turned off ?)\n");
+      fatal(MYNAME ": This is not a MTK based GPS ! (or is device turned off ?)\n");
 
+    // say hello to GR245 to make it display "USB PROCESSING"
+    if (strstr(model, "GR-245")) {
+      holux245_init();	// remember we have a GR245 for mtk_rd_deinit()
+      rc |= do_cmd("$PHLX810*35\r\n", "PHLX852,", NULL, 10);
+      rc |= do_cmd("$PHLX826*30\r\n", "PHLX859*38", NULL, 10);
+      if (rc != 0)
+        dbg(2, "Greeting not successfull.\n");
+    }
+    xfree(model);
  }
 
 static void mtk_rd_deinit(void){
+    if (mtk_device == HOLUX_GR245) {
+      int rc = do_cmd("$PHLX827*31\r\n", "PHLX860*32", NULL, 10);
+      if (rc != 0)
+        dbg(2, "Goodbye not successfull.\n");
+    }
+
     dbg(3, "Closing port...\n");
     gbser_deinit(fd);
     fd = NULL;
@@ -435,17 +481,34 @@ static void mtk_read(void){
   char cmd[256];
   char *line = NULL;
   unsigned char crc, *data = NULL;
-  int cmdLen, j, bsize, i, len, ff_len, rc, init_scan, retry_cnt, log_enabled;
+  int cmdLen, j, bsize, i, len, ff_len, null_len, rc, init_scan, retry_cnt, log_enabled;
   unsigned int line_size, data_size, data_addr, addr, addr_max;
+  unsigned long dsize, dpos = 0;
   FILE *dout;
   char *fusage = NULL;
 
+
+   if ( *OPT_erase_only != '0' ) {
+      mtk_erase();
+      return;
+   } 
+
   log_enabled = 0;
   init_scan = 0;
-  dout = fopen(TEMP_DATA_BIN, "wb");
+  dout = fopen(TEMP_DATA_BIN, "r+b");
   if ( dout == NULL ){
-     fatal(MYNAME ": Can't create temporary file %s", TEMP_DATA_BIN);
-     return;
+     dout = fopen(TEMP_DATA_BIN, "wb");
+     if ( dout == NULL ){
+        fatal(MYNAME ": Can't create temporary file %s", TEMP_DATA_BIN);
+        return;
+     }
+  }
+  fseek(dout, 0L,SEEK_END);
+  dsize = ftell(dout);
+  if ( dsize > 1024 ){
+     dbg(1, "Temp %s file exists. with size %d\n", TEMP_DATA_BIN, dsize);
+     dpos = 0;
+     init_scan = 1;
   }
   dbg(1, "Download %s -> %s\n", port, TEMP_DATA_BIN);
 
@@ -476,9 +539,24 @@ static void mtk_read(void){
    }  
   
   if ( addr_max == 0 ) { // get flash usage failed...      
-    addr_max = 0x200000-64*1024;  // 16Mbit/2Mbyte/32x64kByte block. -- fixme Q1000-ng has 32Mbit 
+    addr_max = 0x200000;  // 16Mbit/2Mbyte/32x64kByte block. -- fixme Q1000-ng has 32Mbit 
     init_scan = 1;
   }
+  dbg(1, "Download %dkB from device\n", (addr_max+1) >> 10);
+
+  if ( dsize > addr_max ){
+     dbg(1, "Temp %s file (%ld) is larger than data size %d. Data erased since last download !\n", TEMP_DATA_BIN, dsize, addr_max);
+     fclose(dout);
+     dsize = 0;
+     init_scan = 0;
+     rename(TEMP_DATA_BIN, TEMP_DATA_BIN_OLD);
+     dout = fopen(TEMP_DATA_BIN, "wb");
+     if ( dout == NULL ){
+        fatal(MYNAME ": Can't create temporary file %s", TEMP_DATA_BIN);
+        return;
+     }
+  }
+
   bsize = 0x0400;
   addr  = 0x0000;
 
@@ -528,21 +606,36 @@ mtk_retry:
                i = 20;
                j = 0;
                ff_len = 0; // number of 0xff bytes.
+               null_len = 0; // number of 0x00 bytes.
                while ( i < (len-3) ){
                   data[j] = (isdigit(line[i])?(line[i]-'0'):(line[i]-'A'+0xA))*0x10 +
                             (isdigit(line[i+1])?(line[i+1]-'0'):(line[i+1]-'A'+0xA));
                   if ( data[j] == 0xff )
                      ff_len++;
+                  if ( data[j] == 0x00 )
+                     null_len++;
                   i += 2;
                   j++;          
                }
                if ( init_scan ){
-                  if ( ff_len == j ){ // data in sector - we've found max sector..
+                  if ( dsize > 0 && addr < dsize ){
+                    fseek(dout, addr, SEEK_SET);
+                    if ( fread(line, 1, bsize, dout) == bsize && memcmp(line, data, bsize) == 0 ){
+                       dpos = addr;
+                       dbg(2, "%s same at %d\n", TEMP_DATA_BIN, addr);
+                    } else {
+                       dbg(2, "%s differs at %d\n", TEMP_DATA_BIN, addr);
+                    }   
+                 }
+                 if ( ff_len == j ){ // data in sector - we've found max sector..
                      addr_max = addr;
-                     dbg(3, "### Init scan complete - max block is 0x%6x!!! ###\n", addr_max);
+                     dbg(1, "Initial scan done - Download %dkB from device\n", (addr_max+1) >> 10);
                   }   
                   len = 0;
                } else {
+                  if ( null_len == j ){ // 0x00 block - bad block....
+                     fprintf(stderr, "FIXME -- read bad block at 0x%.6x - retry ? skip ?\n%s\n", data_addr, line);
+                  }
                   if ( fwrite(data, 1, j, dout) != j )
                      fatal(MYNAME ": Failed to write temp. binary file\n");
                   if ( ff_len == j ){ // 0xff block - read complete...
@@ -553,29 +646,41 @@ mtk_retry:
                      len = 0;
                   }
                }    
-            } 
+            } else if ( strncmp(line, "$PMTK001,182,7,", 15) == 0 ){ // Command ACK
+               if ( line[15] != '3' ) {
+                  // fixme - we should timeout here when no log data has been received...
+                  dbg(2, "\nLog req. failed (%c)\n", line[15]);
+                  gb_sleep(10*1000);
+                  retry_cnt++;
+                  goto mtk_retry;
+               }   
+            }
          }
       } while ( len != 0 );
       if ( init_scan ){
          addr += 0x10000;
          if ( addr >= addr_max ){ // initial scan complete...
             init_scan = 0;
-            addr = 0x0000;
+            addr = dpos;
          }
       } else {
-         int perc;
          addr += bsize;
-         perc = 100 - 100*(addr_max-addr)/addr_max;
-         if ( addr >= addr_max ) 
-           perc = 100;
-         dbg(2, "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\bReading 0x%.6x %3d %%", addr, perc);
+         if ( global_opts.verbose_status || (global_opts.debug_level >= 2 && global_opts.debug_level < 5) ){
+            int perc;
+            perc = 100 - 100*(addr_max-addr)/addr_max;
+            if ( addr >= addr_max ) 
+              perc = 100;
+            fprintf(stderr, "\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\bReading 0x%.6x %3d %%", addr, perc);
+         }
       }
   }
   if ( dout != NULL )
      fclose(dout);
-   dbg(2, "\n");
+   if ( global_opts.verbose_status || (global_opts.debug_level >= 2 && global_opts.debug_level < 5) )
+      fprintf(stderr,"\n");
 
-  if ( log_enabled ){
+  // Fixme - Order or. Enable - parse - erase ??
+  if ( log_enabled || *OPT_log_enable=='1' ){
      i = do_cmd(CMD_LOG_ENABLE, "PMTK001,182,4,3", NULL, 2);
      dbg(3, " ---- LOG ENABLE ----%s\n", i==0?"Success":"Fail");
   } else {
@@ -591,7 +696,7 @@ mtk_retry:
   file_deinit();   
  
    /* fixme -- we're assuming all went well - erase flash.... */
-   if ( *erase != '0' ) {
+   if ( *OPT_erase != '0' ) {
       mtk_erase();
    } 
      
@@ -674,8 +779,12 @@ static int add_trackpoint(int idx, unsigned long bmask, struct data_item *itm){
        trk->sat = itm->sat_used;
 
     // RCR is a bitmask of possibly several log reasons..
+    // Holux devics use a Event prefix for each waypt. 
     if ( global_opts.masked_objective & WPTDATAMASK
-       && bmask & (1<<RCR) && itm->rcr & 0x0008  )
+       && (   (bmask & (1<<RCR) && itm->rcr & 0x0008) 
+            || (mtk_info.track_event & MTK_EVT_WAYPT)
+          )
+       )
     {
        /* Button press -- create waypoint, start count at 1 */
        waypoint *w = waypt_dupe(trk);
@@ -854,8 +963,8 @@ static int csv_line(gbfile *csvFile, int idx, unsigned long bmask, struct data_i
 /********************* MTK Logger -- Parse functions *********************/
 int mtk_parse(unsigned char *data, int dataLen, unsigned int bmask){
    static int count = 0;
-   int i, k, sat_id;
-   unsigned char crc;
+   int i, k, sat_id, hspd;
+   unsigned char crc, hbuf[4];
    struct data_item itm;
 
    dbg(5,"Entering mtk_parse, count = %i, dataLen = %i\n", count, dataLen);
@@ -894,19 +1003,34 @@ int mtk_parse(unsigned char *data, int dataLen, unsigned int bmask){
             }
             break;
         case 1<<HEIGHT:
-            if ( is_m241 ) {
-               unsigned char tmp[4];
-               tmp[0] = 0x0;
-               tmp[1] = *(data + i);
-               tmp[2] = *(data + i + 1);
-               tmp[3] = *(data + i + 2);
-               itm.height = endian_read_float(tmp, 1 /* le */);
-            } else {
-               itm.height = endian_read_float(data + i, 1 /* le */);          
+            switch ( mtk_device ){
+               case HOLUX_GR245: // Stupid Holux GPsport 245 - log speed as centimeters/sec. (in height position !) 
+                  hspd = data[i] + data[i+1]*0x100 + data[i+2]*0x10000 + data[i+3]*0x1000000;
+                  itm.speed =  MPS_TO_KPH(hspd)/100.; // convert to km/h..
+                  break;
+               case HOLUX_M241:
+                  hbuf[0] = 0x0;
+                  hbuf[1] = *(data + i);
+                  hbuf[2] = *(data + i + 1);
+                  hbuf[3] = *(data + i + 2);
+                  itm.height = endian_read_float(hbuf, 1 /* le */);
+                  break;
+                case MTK_LOGGER:
+                default:
+                  itm.height = endian_read_float(data + i, 1 /* le */);
+                  break;
             }
             break;
          case 1<<SPEED: 
-            itm.speed = endian_read_float(data + i, 1 /* le */);
+            if ( mtk_device == HOLUX_GR245 ){ // Stupid Holux GPsport 245 - log height in speed position...
+               hbuf[0] = 0x0;
+               hbuf[1] = *(data + i);
+               hbuf[2] = *(data + i + 1);
+               hbuf[3] = *(data + i + 2);
+               itm.height = endian_read_float(hbuf, 1 /* le */);
+            } else {
+               itm.speed = endian_read_float(data + i, 1 /* le */);
+            }
             break;
          case 1<<HEADING: 
             itm.heading = endian_read_float(data + i, 1 /* le */);
@@ -989,6 +1113,10 @@ int mtk_parse(unsigned char *data, int dataLen, unsigned int bmask){
          case 1<<DISTANCE:         
             itm.distance = endian_read_double(data + i, 1 /* le */);
             break;
+         default:
+             // if ( ((1<<k) & bmask) ) 
+             //   printf("Unknown ID %d: %.2x %.2x %.2x %.2x\n", k, data[i], data[i+1], data[i+2], data[i+3]);
+             break;           
       } /* End: switch (bmap) */
 
       /* update item checksum and length */
@@ -1000,12 +1128,17 @@ int mtk_parse(unsigned char *data, int dataLen, unsigned int bmask){
       }
    } /* for (bmap,...) */
  
-   if ( ! is_m241 ){
-      if ( data[i] == '*' )
+   if ( mtk_device == MTK_LOGGER ){  // Holux skips '*' checksum separator
+      if ( data[i] == '*' ){
          i++; // skip '*' separator
-      else 
-         dbg(1,"Missing '*' !\n");   
-   }   
+      } else {
+         dbg(1,"Missing '*' !\n");
+         if ( data[i] == 0xff ){ // in some case star-crc hasn't been written on power off.
+            dbg(1, "Bad data point @0x%.6x - skip %d bytes\n", (fl!=NULL)?ftell(fl):-1, i+2);
+            return i+2; // include '*' and crc
+         }
+      }
+   }
    if ( memcmp(&data[0], &LOG_RST[0], 6) == 0 
         && memcmp(&data[12], &LOG_RST[12], 4) == 0  )
    {
@@ -1015,7 +1148,7 @@ int mtk_parse(unsigned char *data, int dataLen, unsigned int bmask){
    }    
 
    if ( data[i] != crc ){
-      dbg(0,"%2d: Bad CRC %.2x != %.2x\n", count, data[i], crc);
+      dbg(0,"%2d: Bad CRC %.2x != %.2x (pos 0x%.6x)\n", count, data[i], crc, (fl!=NULL)?ftell(fl):-1);
    }
    i++; // crc         
    count++;
@@ -1047,8 +1180,10 @@ static int mtk_parse_info(const unsigned char *data, int dataLen){
          case 0x02:
             bm = le_read32(data + 8);
             dbg(1, "# Log bitmask is: %.8x\n", bm);
-            if ( is_m241 ) 
+            if ( mtk_device != MTK_LOGGER ) 
                bm &= 0x7fffffffU;
+            if ( mtk_device == HOLUX_GR245 )
+               bm &= ~HOLUX245_MASK;
             if ( mtk_info.bitmask != bm ){
                dbg(1," ########## Bitmask Change   %.8x -> %.8x ###########\n", mtk_info.bitmask, bm);
                mtk_info.track_event |= MTK_EVT_BITMASK;
@@ -1093,7 +1228,7 @@ static int mtk_parse_info(const unsigned char *data, int dataLen){
    } else {
       if ( global_opts.debug_level > 0 ){
         fprintf(stderr,"#!! Invalid INFO block !! %d bytes\n >> ", dataLen);
-        for (bm=0;bm<16;bm++) printf("%.2x ", data[bm]);
+        for (bm=0;bm<16;bm++) fprintf(stderr, "%.2x ", data[bm]);
         fprintf(stderr,"\n");
       }
       return 0;
@@ -1105,9 +1240,20 @@ static int mtk_log_len(unsigned int bitmask){
    int i, len;
    
    /* calculate the length of a binary log item. */
-   len = 2; // add '*' + crc, holux would only be +1, oh, well...
+    switch ( mtk_device ){
+       case HOLUX_M241:
+       case HOLUX_GR245:    
+          len = 1; // add crc 
+          break;
+       case MTK_LOGGER:
+       default:
+         len = 2; // add '*' + crc
+         break;
+   }
    for (i=0;i<32;i++){
       if ( (1<<i) & bitmask ){
+         if ( i > DISTANCE && global_opts.debug_level > 0 )
+            warning(MYNAME ": Unknown size/meaning of bit %d\n", i);
          if ( (i == SID || i == ELEVATION || i == AZIMUTH || i == SNR) && (1<<SID) & bitmask )
             len += log_type[i].size*32; // worst case, max sat. count..
          else
@@ -1121,7 +1267,7 @@ static int mtk_log_len(unsigned int bitmask){
 /********************** File-in interface ********************************/
 
 static void file_init_m241(const char *fname) {
-    is_m241=1;
+    mtk_device = HOLUX_M241;
     file_init(fname);
 }
 
@@ -1130,9 +1276,14 @@ static void file_init(const char *fname) {
     if (fl = fopen(fname, "rb"), NULL == fl) {
         fatal(MYNAME ": Can't open file '%s'\n", fname);
     }
-    if ( is_m241 ) {
-       log_type[LATITUDE].size = log_type[LONGITUDE].size = 4;
-       log_type[HEIGHT].size = 3;
+    switch (mtk_device){
+       case HOLUX_M241:
+       case HOLUX_GR245:
+          log_type[LATITUDE].size = log_type[LONGITUDE].size = 4;
+          log_type[HEIGHT].size = 3;
+       break;
+       default:
+        break;
     }
 }
 
@@ -1141,8 +1292,19 @@ static void file_deinit(void) {
     fclose(fl);
 }
 
+static void holux245_init(void){
+    mtk_device = HOLUX_GR245;
+
+    // stupid workaround for a broken Holux-245 device....
+    // Height & speed have changed position in bitmask and data on Holux 245 Argh !!!
+    log_type[HEIGHT].id   = SPEED;
+    log_type[HEIGHT].size = 4; // speed size - unit: cm/sec
+    log_type[SPEED].id    = HEIGHT;
+    log_type[SPEED].size  = 3; // height size..
+}
+
 static int is_holux_string(const unsigned char *data, int dataLen) {
-   if ( is_m241 &&
+   if ( mtk_device != MTK_LOGGER &&
         dataLen >= 5 &&
         data[0] == (0xff & 'H') &&
         data[1] == (0xff & 'O') &&
@@ -1187,15 +1349,19 @@ static void file_read(void) {
    /* get default bitmask, log period/speed/distance */
    bLen = fread(buf, 1, 20, fl);
    if ( bLen == 20 ){
-      unsigned int mask, log_period, log_distance, log_speed;
-       
+      unsigned int mask, log_period, log_distance, log_speed, log_policy;
+      log_policy   = le_read16(buf + 6);
+
+      if ( !(log_policy == 0x0104 || log_policy == 0x0106) && fsize > 0x10000 ){
+         dbg(1, "Invalid initial log policy 0x%.4x - check next block\n", log_policy);
+         fseek(fl, 0x10000, SEEK_SET);
+         bLen = fread(buf, 1, 20, fl);
+         log_policy   = le_read16(buf + 6);
+      }       
       mask = le_read32(buf + 2);
-      if ( is_m241 ) {
-         // clear Holux-specific 'low precision' bit
+      if ( mtk_device != MTK_LOGGER ) { // clear Holux-specific 'low precision' bit
          mask &= 0x7fffffffU;
       }
-
-      // log_policy   = le_read16(buf + 6);
       log_period   = le_read32(buf + 8);
       log_distance = le_read32(buf + 12);
       log_speed    = le_read32(buf + 16);
@@ -1210,10 +1376,10 @@ static void file_read(void) {
       mtk_info.speed  = log_speed;
    }
    mtk_info.track_event = 0;
-   
+
    pos = 0x200; // skip header...first data position 
    fseek(fl, pos, SEEK_SET);  
-   
+
    /* read initial info blocks -- if any */   
    do {
       bLen = fread(buf, 1, 16, fl);
@@ -1223,6 +1389,7 @@ static void file_read(void) {
         pos += j;
       } else if  ( is_holux_string(buf, bLen) ) {
 	pos += j;
+        // Note -- Holux245 will have <SP><SP><SP><SP> here...handled below..
       }
    } while ( j == 16 );
    j = bLen;
@@ -1244,9 +1411,23 @@ static void file_read(void) {
             mtk_parse_info(&buf[i], (bLen-i));
             k = 16;
          } else if  ( is_holux_string(&buf[i], (bLen - i)) ) {
+            if ( memcmp(&buf[i+10], "WAYPNT", 6) == 0 )
+               mtk_info.track_event |= MTK_EVT_WAYPT;
+
             k = 16;
+            // m241  - HOLUXGR241LOGGER or HOLUXGR241WAYPNT or HOLUXGR241LOGGER<SP><SP><SP><SP>
+            // gr245 - HOLUXGR245LOGGER<SP><SP><SP><SP> or HOLUXGR245WAYPNT<SP><SP><SP><SP>
+            if (memcmp(&buf[i], "HOLUXGR245", 10) == 0) {
+               dbg(2, "Detected Holux GR245 !\n");
+               holux245_init();
+            }
+
+            // skip the 4 spaces that may occur on every device
+            if ( memcmp(&buf[i+16], "    ", 4) == 0 ){ // Assume loglen >= 20...
+               k += 4;
+            }   
          } else if  ( buf[i] == 0xff && buf[i+1] == 0xff  && buf[i+2] == 0xff && buf[i+3] == 0xff
-               /* && (pos + 2*logLen) & 0xffff) < logLen */)
+                /* && ((pos + 2*mtk_info.logLen) & 0xffff) < mtk_info.logLen */  )
          {
             /* End of 64k block segment -- realign to next data area */
 
@@ -1331,7 +1512,7 @@ static arglist_t mtk_fargs[] = {
 
 ff_vecs_t mtk_fvecs = {
     ff_type_file,
-    { ff_cap_none, ff_cap_read, ff_cap_none },
+    { ff_cap_read, ff_cap_read, ff_cap_none },
     file_init,
     NULL,
     file_deinit,
@@ -1345,7 +1526,7 @@ ff_vecs_t mtk_fvecs = {
 
 ff_vecs_t mtk_m241_fvecs = {
     ff_type_file,
-    { ff_cap_none, ff_cap_read, ff_cap_none },
+    { ff_cap_read, ff_cap_read, ff_cap_none },
     file_init_m241,
     NULL,
     file_deinit,
