@@ -173,14 +173,17 @@ static char *getposnarg;
 static char *opt_sleep;
 static char *opt_baud;
 static char *opt_append;
+static char *opt_gisteq;
 
 static long sleepus;
 static int getposn;
 static int append_output;
+static int amod_waypoint;
 
 static time_t last_time;
 static double last_read_time;   /* Last timestamp of GGA or PRMC */
 static int datum;
+static int had_checksum;
 
 static waypoint * nmea_rd_posn(posn_status *);
 static void nmea_rd_posn_init(const char *fname);
@@ -197,6 +200,7 @@ arglist_t nmea_args[] = {
 	{"pause", &opt_sleep, "Decimal seconds to pause between groups of strings", NULL, ARGTYPE_INT, ARG_NOMINMAX },
 	{"append_positioning", &opt_append, "Append realtime positioning data to the output file instead of truncating", "0", ARGTYPE_BOOL, ARG_NOMINMAX },
 	{"baud", &opt_baud, "Speed in bits per second of serial port (baud=4800)", NULL, ARGTYPE_INT, ARG_NOMINMAX },
+	{"gisteq", &opt_gisteq, "Write tracks for Gisteq Phototracker", "0", ARGTYPE_BOOL, ARG_NOMINMAX },
 	ARG_TERMINATOR
 };
 
@@ -232,6 +236,17 @@ nmea_add_wpt(waypoint *wpt, route_head *trk)
 	else waypt_add(wpt);
 }
 
+static void
+nmea_release_wpt(waypoint *wpt)
+{
+	if (wpt && ((wpt->Q.next == NULL) || (wpt->Q.next == &wpt->Q))) {
+		/* This waypoint isn't queued.
+		   Release it, because we don't have any reference to this
+		   waypoint (! memory leak !) */
+		waypt_free(wpt);
+	}
+}
+
 static void
 nmea_rd_init(const char *fname)
 {
@@ -239,11 +254,13 @@ nmea_rd_init(const char *fname)
 	last_waypt = NULL;
 	last_time = -1;
 	datum = DATUM_WGS84;
+	had_checksum = 0;
 
 	CHECK_BOOL(opt_gprmc);
 	CHECK_BOOL(opt_gpgga);
 	CHECK_BOOL(opt_gpvtg);
 	CHECK_BOOL(opt_gpgsa);
+	CHECK_BOOL(opt_gisteq);
 
 	QUEUE_INIT(&pcmpt_head);
 
@@ -298,6 +315,7 @@ nmea_wr_init(const char *portname)
 	CHECK_BOOL(opt_gpgga);
 	CHECK_BOOL(opt_gpvtg);
 	CHECK_BOOL(opt_gpgsa);
+	CHECK_BOOL(opt_gisteq);
 
 	append_output = atoi(opt_append);
 
@@ -313,8 +331,14 @@ nmea_wr_init(const char *portname)
 		}
 	}
 
-	mkshort_handle = mkshort_new_handle();
+	mkshort_handle = (struct short_handle*) mkshort_new_handle();
 	setshort_length(mkshort_handle, atoi(snlenopt));
+
+	if (opt_gisteq) {
+		opt_gpgga = NULL;
+		opt_gpvtg = NULL;
+		opt_gpgsa = NULL;
+	}
 }
 
 static  void
@@ -325,23 +349,25 @@ nmea_wr_deinit(void)
 }
 
 static void
-nmea_set_waypoint_time(waypoint *wpt, struct tm *time)
+nmea_set_waypoint_time(waypoint *wpt, struct tm *time, int microseconds)
 {
 	if (time->tm_year == 0)
 	{
 		wpt->creation_time = ((((time_t)time->tm_hour * 60) + time->tm_min) * 60) + time->tm_sec;
-		if (wpt->microseconds == 0)
+		wpt->microseconds = microseconds;
+		if (wpt->wpt_flags.fmt_use == 0)
 		{
-			 wpt->microseconds++;
+			 wpt->wpt_flags.fmt_use = 1;
 			 without_date++;
 		}
 	}
 	else
 	{
 		wpt->creation_time = mkgmtime(time);
-		if (wpt->microseconds != 0)
+		wpt->microseconds = microseconds;
+		if (wpt->wpt_flags.fmt_use != 0)
 		{
-			wpt->microseconds = 0;
+			wpt->wpt_flags.fmt_use = 0;
 			without_date--;
 		}
 	}
@@ -350,8 +376,9 @@ nmea_set_waypoint_time(waypoint *wpt, struct tm *time)
 static void
 gpgll_parse(char *ibuf)
 {
-	double latdeg, lngdeg;
+	double latdeg, lngdeg, microseconds;
 	char lngdir, latdir;
+	double hmsd;
 	int hms;
 	char valid = 0;
 	waypoint *waypt;
@@ -361,14 +388,17 @@ gpgll_parse(char *ibuf)
 		track_add_head(trk_head);
 	}
 
-	sscanf(ibuf,"$GPGLL,%lf,%c,%lf,%c,%d,%c,",
+	sscanf(ibuf,"$GPGLL,%lf,%c,%lf,%c,%lf,%c,",
 		&latdeg,&latdir,
 		&lngdeg,&lngdir,
-		&hms,&valid);
+		&hmsd,&valid);
 
 	if (valid != 'A')
 		return;
 
+	hms = (int) hmsd;
+	microseconds = MILLI_TO_MICRO(1000 * (hmsd - hms));
+
 	tm.tm_sec = hms % 100;
 	hms = hms / 100;
 	tm.tm_min = hms % 100;
@@ -379,7 +409,7 @@ gpgll_parse(char *ibuf)
 
 	waypt = waypt_new();
 
-	nmea_set_waypoint_time(waypt, &tm);
+	nmea_set_waypoint_time(waypt, &tm, microseconds);
 
 	if (latdir == 'S') latdeg = -latdeg;
 	waypt->latitude = ddmm2degrees(latdeg);
@@ -387,9 +417,7 @@ gpgll_parse(char *ibuf)
 	if (lngdir == 'W') lngdeg = -lngdeg;
 	waypt->longitude = ddmm2degrees(lngdeg);
 
-	if (curr_waypt && (read_mode == rm_serial)) {
-		waypt_free(curr_waypt);
-	}
+	nmea_release_wpt(curr_waypt);
 	curr_waypt = waypt;
 }
 
@@ -405,6 +433,7 @@ gpgga_parse(char *ibuf)
 	double hdop;
 	char altunits;
 	waypoint *waypt;
+	double microseconds;
 
 	if (trk_head == NULL) {
 		trk_head = route_head_alloc();
@@ -421,11 +450,12 @@ gpgga_parse(char *ibuf)
 	 * as serial units will often spit a remembered position up and
 	 * that is more comfortable than nothing at all...
 	 */
-	if ((fix == 0) && (read_mode != rm_serial)) {
+	if ((fix <= 0) && (read_mode != rm_serial)) {
 		return;
 	}
 
 	last_read_time = hms;
+	microseconds = MILLI_TO_MICRO(1000 * (hms - (int)hms));
 
 	tm.tm_sec = (long) hms % 100;
 	hms = hms / 100;
@@ -435,7 +465,7 @@ gpgga_parse(char *ibuf)
 
 	waypt  = waypt_new();
 
-	nmea_set_waypoint_time(waypt, &tm);
+	nmea_set_waypoint_time(waypt, &tm, microseconds);
 
 	if (latdir == 'S') latdeg = -latdeg;
 	waypt->latitude = ddmm2degrees(latdeg);
@@ -464,9 +494,7 @@ gpgga_parse(char *ibuf)
 			break;
 	}
 
-	if (curr_waypt && (read_mode == rm_serial)) {
-		waypt_free(curr_waypt);
-	}
+	nmea_release_wpt(curr_waypt);
 	curr_waypt = waypt;
 }
 
@@ -480,23 +508,40 @@ gprmc_parse(char *ibuf)
 	unsigned int dmy;
 	double speed,course;
 	waypoint *waypt;
+	double microseconds;
+	char *dmybuf;
+	int i;
 
 	if (trk_head == NULL) {
 		trk_head = route_head_alloc();
 		track_add_head(trk_head);
 	}
 
-	sscanf(ibuf,"$GPRMC,%lf,%c,%lf,%c,%lf,%c,%lf,%lf,%u",
+	/*
+	 * Read everything except the dmy, in case lngdeg
+	 * and lngdir are missing.
+	 */
+	sscanf(ibuf,"$GPRMC,%lf,%c,%lf,%c,%lf,%c,%lf,%lf",
 		&hms, &fix, &latdeg, &latdir,
 		&lngdeg, &lngdir,
-		&speed, &course, &dmy);
+		&speed, &course);
 
 	if (fix != 'A') {
 		/* ignore this fix - it is invalid */
 		return;
 	}
 
+	/* Skip past nine commas in ibuf to reach the dmy value */
+	for (dmybuf=ibuf,i=0; i<9 && dmybuf != NULL; i++) {
+		dmybuf= strchr(dmybuf, ',');
+		dmybuf++;
+	}
+
+	/* Now read dmy from the correct position */
+	sscanf(dmybuf,"%u", &dmy);
+
 	last_read_time = hms;
+	microseconds = MILLI_TO_MICRO(1000 * (hms - (int)hms));
 	
 	tm.tm_sec = (long) hms % 100;
 	hms = hms / 100;
@@ -519,8 +564,13 @@ gprmc_parse(char *ibuf)
 				WAYPT_SET(curr_waypt, course, course);
 			/* The change of date wasn't recorded when 
 			 * going from 235959 to 000000. */
-			 nmea_set_waypoint_time(curr_waypt, &tm);
+			 nmea_set_waypoint_time(curr_waypt, &tm, microseconds);
 		}
+                /* This point is both a waypoint and a trackpoint. */
+                if (amod_waypoint) {
+			waypt_add(waypt_dupe(curr_waypt));
+			amod_waypoint = 0;
+                }
 		return;
 	}
 		
@@ -530,7 +580,7 @@ gprmc_parse(char *ibuf)
 
 	WAYPT_SET(waypt, course, course);
 	
-	nmea_set_waypoint_time(waypt, &tm);
+	nmea_set_waypoint_time(waypt, &tm, microseconds);
 
 	if (latdir == 'S') latdeg = -latdeg;
 	waypt->latitude = ddmm2degrees(latdeg);
@@ -538,10 +588,14 @@ gprmc_parse(char *ibuf)
 	if (lngdir == 'W') lngdeg = -lngdeg;
 	waypt->longitude = ddmm2degrees(lngdeg);
 
-	if (curr_waypt && (read_mode == rm_serial)) {
-		waypt_free(curr_waypt);
-	}
+	nmea_release_wpt(curr_waypt);
 	curr_waypt = waypt;
+
+	/* This point is both a waypoint and a trackpoint. */
+	if (amod_waypoint) {
+		waypt_add(waypt_dupe(waypt));
+		amod_waypoint = 0;
+	}
 }
 
 static void
@@ -728,7 +782,7 @@ pcmpt_parse(char *ibuf)
 		tm.tm_mon  = dmy % 100 - 1;
 		dmy = dmy / 100;
 		tm.tm_mday = dmy;
-		nmea_set_waypoint_time(curr_waypt, &tm);
+		nmea_set_waypoint_time(curr_waypt, &tm, 0);
 		ENQUEUE_HEAD(&pcmpt_head, &curr_waypt->Q);
 	} else {
 		queue *elem, *tmp;
@@ -802,11 +856,11 @@ nmea_fix_timestamps(route_head *track)
 		{
 			waypoint *wpt = (waypoint *)elem;
 			
-			if (wpt->microseconds != 0)
+			if (wpt->wpt_flags.fmt_use != 0)
 			{
 				time_t dt;
 				
-				wpt->microseconds = 0;		/* reset flag */
+				wpt->wpt_flags.fmt_use = 0;	/* reset flag */
 
 				dt = (prev / SECONDS_PER_DAY) * SECONDS_PER_DAY;
 				wpt->creation_time += dt;
@@ -821,7 +875,6 @@ nmea_fix_timestamps(route_head *track)
 void
 nmea_parse_one_line(char *ibuf)
 {
-	int had_checksum = 0;
 	char *ck;
 	int ckval, ckcmp;
 	char *tbuf = lrtrim(ibuf);
@@ -901,7 +954,10 @@ nmea_parse_one_line(char *ibuf)
 	} else
 	if (opt_gpgsa && (0 == strncmp(tbuf, "$GPGSA,",7))) {
 		gpgsa_parse(tbuf); /* GPS fix */
-	}
+	} else
+        if (0 == strncmp(tbuf, "$ADPMB,5,0", 10)) {
+          	amod_waypoint = 1;
+        }
 
 	if (tbuf != ibuf) {
 	  /* clear up the dynamic buffer we used because substition was required */
@@ -946,15 +1002,27 @@ nmea_read(void)
 		
 		line++;
 		
+		if ((line == 0) & file_in->unicode) cet_convert_init(CET_CHARSET_UTF8, 1);
+
 		if ((line == 0) && (case_ignore_strncmp(ibuf, "@SonyGPS/ver", 12) == 0)) {
 			/* special hack for Sony GPS-CS1 files:
 			   they are fully (?) nmea compatible, but come with a header line like
 			   "@Sonygps/ver1.0/wgs-84". */
+			/* The Sony GPS-CS3KA extends that line even further
+			   so we now look for the second field to be / 
+			   delimited.
+			   @Sonygps/ver1.0/wgs-84/gps-cs3.0
+			 */
 			   
 			/* Check the GPS datum */
 			cx = strchr(&ibuf[12], '/');
 			if (cx != NULL) {
+				char *edatum;
 				sdatum = cx + 1;
+			 	edatum = strchr(sdatum, '/');
+				if (edatum) {
+					*edatum = 0;
+				}
 				datum = GPS_Lookup_Datum_Index(sdatum);
 				if (datum < 0)
 					fatal(MYNAME "/SonyGPS: Unsupported datum \"%s\" in source data!\n", sdatum);
@@ -1198,8 +1266,8 @@ nmea_trackpt_pr(const waypoint *wpt)
 	}
 
 	if (opt_gprmc) {
-		snprintf(obuf, sizeof(obuf), "GPRMC,%06d,%c,%08.3f,%c,%09.3f,%c,%.2f,%.2f,%06d,,",
-				(int) hms,
+		snprintf(obuf, sizeof(obuf), "GPRMC,%010.3f,%c,%08.3f,%c,%09.3f,%c,%.2f,%.2f,%06d,,",
+				(double) hms + (wpt->microseconds / 1000000.0),
 				fix=='0' ? 'V' : 'A',
 				fabs(lat), lat < 0 ? 'S' : 'N',
 				fabs(lon), lon < 0 ? 'W' : 'E',
@@ -1207,11 +1275,18 @@ nmea_trackpt_pr(const waypoint *wpt)
 				WAYPT_HAS(wpt, course) ? (wpt->course):(0),
 				(int) ymd);
 		cksum = nmea_cksum(obuf);
+
+		/* GISTeq doesn't care about the checksum, but wants this prefixed, so
+		 * we can write it with abandon.
+  		 */
+		if (opt_gisteq) {
+			gbfprintf(file_out, "---,");
+		}
 		gbfprintf(file_out, "$%s*%02X\n", obuf, cksum);
 	}
 	if (opt_gpgga) {
-		snprintf(obuf, sizeof(obuf), "GPGGA,%06d,%08.3f,%c,%09.3f,%c,%c,%02d,%.1f,%.3f,M,0.0,M,,",
-				(int) hms,
+		snprintf(obuf, sizeof(obuf), "GPGGA,%010.3f,%08.3f,%c,%09.3f,%c,%c,%02d,%.1f,%.3f,M,0.0,M,,",
+				(double) hms + (wpt->microseconds / 1000000.0),
 				fabs(lat), lat < 0 ? 'S' : 'N',
 				fabs(lon), lon < 0 ? 'W' : 'E',
 				fix,
@@ -1254,6 +1329,7 @@ nmea_trackpt_pr(const waypoint *wpt)
 		cksum = nmea_cksum(obuf);
 		gbfprintf(file_out, "$%s*%02X\n", obuf, cksum);
 	}
+        gbfflush(file_out);
 }
 
 static void
@@ -1284,7 +1360,9 @@ nmea_wr_posn_deinit(void)
 
 ff_vecs_t nmea_vecs = {
 	ff_type_file,
-	{ ff_cap_read | ff_cap_write, ff_cap_read | ff_cap_write, ff_cap_none},
+	{ (ff_cap)(ff_cap_read | ff_cap_write),
+          (ff_cap)(ff_cap_read | ff_cap_write), 
+          ff_cap_none},
 	nmea_rd_init,	
 	nmea_wr_init,	
 	nmea_rd_deinit,	
