@@ -795,7 +795,7 @@ osm_waypt_disp(const waypoint *wpt)
 		*id = --node_id;
 		((waypoint *)(wpt))->extra_data = id;
 	
-		gbfprintf(fout, "  <node id='%d' visible='true' lat='%f' lon='%f'", *id, wpt->latitude, wpt->longitude);
+		gbfprintf(fout, "  <node id='%d' visible='true' lat='%0.7f' lon='%0.7f'", *id, wpt->latitude, wpt->longitude);
 		if (wpt->creation_time) {
 			char time_string[64];
 			xml_fill_in_time(time_string, wpt->creation_time, wpt->microseconds, XML_LONG_TIME);
