@@ -1,5 +1,5 @@
 /* 
-    Copyright (C) 2002-2006 Robert Lipe, robertlipe@usa.net
+    Copyright (C) 2002-2010 Robert Lipe, robertlipe@gpsbabel.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -29,6 +29,8 @@ static int rte_waypts;
 static int trk_head_ct;
 static int trk_waypts;
 
+extern void update_common_traits(const waypoint* wpt);
+
 void
 route_init(void)
 {
@@ -135,6 +137,13 @@ track_del_head(route_head *rte)
 	trk_head_ct--;
 }
 
+void
+track_insert_head(route_head *rte, route_head *predecessor)
+{
+	ENQUEUE_AFTER(&predecessor->Q, &rte->Q );
+	trk_head_ct++;
+}
+
 static 
 route_head *
 common_route_by_name(queue *routes, const char *name)
@@ -178,17 +187,32 @@ any_route_add_wpt(route_head *rte, waypoint *wpt, int *ct, int synth )
 		wpt->shortname = xstrdup(tmpnam);
 		wpt->wpt_flags.shortname_is_synthetic = 1;
 	}
+	update_common_traits(wpt);
 }
 
 void 
 route_add_wpt( route_head *rte, waypoint *wpt )
 {
+	// First point in a route is always a new segment.
+	// This improves compatibility when reading from
+	// segment-unaware formats.
+	if (QUEUE_EMPTY(&rte->waypoint_list)) {
+		wpt->wpt_flags.new_trkseg = 1;
+	}
+	
 	any_route_add_wpt( rte, wpt, &rte_waypts, 1 );
 }
 
 void 
 track_add_wpt( route_head *rte, waypoint *wpt )
 {
+	// First point in a track is always a new segment.
+	// This improves compatibility when reading from 
+	// segment-unaware formats.
+	if (QUEUE_EMPTY(&rte->waypoint_list)) {
+		wpt->wpt_flags.new_trkseg = 1;
+	}
+	
 	any_route_add_wpt( rte, wpt, &trk_waypts, 0 );
 }
 
@@ -209,6 +233,11 @@ route_find_waypt_by_name( route_head *rh, const char *name )
 static void 
 any_route_del_wpt( route_head *rte, waypoint *wpt, int *ct)
 {
+	if (wpt->wpt_flags.new_trkseg && wpt != (waypoint*)QUEUE_LAST(&rte->waypoint_list)) {
+		waypoint* wpt_next = (waypoint*)QUEUE_NEXT(&wpt->Q);
+		wpt_next->wpt_flags.new_trkseg = 1;
+	}
+	wpt->wpt_flags.new_trkseg = 0;
 	dequeue( &wpt->Q );
 	rte->rte_waypt_ct--;
 	if ( ct ) (*ct)--;
@@ -236,7 +265,7 @@ route_disp (const route_head *rh, waypt_cb cb )
 	QUEUE_FOR_EACH(&rh->waypoint_list, elem, tmp) {
 		waypoint *waypointp;
 		waypointp = (waypoint *) elem;
-			(*cb)(waypointp);
+		(*cb)(waypointp);
 	}
 		
 }
@@ -360,7 +389,7 @@ route_copy( int *dst_count, int *dst_wpt_count, queue **dst, queue *src ) {
 	}
 	
 	if ( !*dst ) {
-		*dst = xcalloc( 1, sizeof( queue ));
+		*dst = (queue *)xcalloc( 1, sizeof( queue ));
 		QUEUE_INIT( *dst );
 		*dst_count = 0;
 		*dst_wpt_count = 0;
@@ -517,7 +546,7 @@ tracks_to_routes(void)
 void track_recompute(const route_head *trk, computed_trkdata **trkdatap)
 {
 	waypoint first;
-	waypoint *this;
+	waypoint *thisw;
 	waypoint *prev = &first;
 	queue *elem, *tmp;
 	int tkpt = 0;
@@ -526,7 +555,7 @@ void track_recompute(const route_head *trk, computed_trkdata **trkdatap)
 	int pts_cad = 0;
 	double tot_cad = 0.0;
 	char tkptname[100];
-	computed_trkdata *tdata = xcalloc(1, sizeof (computed_trkdata));
+	computed_trkdata *tdata = (computed_trkdata *)xcalloc(1, sizeof (computed_trkdata));
 
 	if (trkdatap) {
 		*trkdatap = tdata;
@@ -543,17 +572,17 @@ void track_recompute(const route_head *trk, computed_trkdata **trkdatap)
 		time_t timed;
 		double tlat, tlon, plat, plon, dist;
 
-		this = (waypoint *)elem;
-		timed = this->creation_time - prev->creation_time;
+		thisw = (waypoint *)elem;
+		timed = thisw->creation_time - prev->creation_time;
 
 		/* 
 		 * gcdist and heading want radians, not degrees.
 		 */
-		tlat = RAD(this->latitude);
-		tlon = RAD(this->longitude);
+		tlat = RAD(thisw->latitude);
+		tlon = RAD(thisw->longitude);
 		plat = RAD(prev->latitude);
 		plon = RAD(prev->longitude);
-		WAYPT_SET(this, course, heading_true_degrees(plat, plon,
+		WAYPT_SET(thisw, course, heading_true_degrees(plat, plon,
 			tlat, tlon));
 		dist = radtometers(gcdist(plat, plon, tlat, tlon));
 
@@ -565,62 +594,67 @@ void track_recompute(const route_head *trk, computed_trkdata **trkdatap)
 		}
 
 		/*
-		 * If we've moved as much as a meter, recompute speed.
+		 * If we've moved as much as a meter, 
+		 * conditionally recompute speeds.
 		 */
 		if (timed && (dist > 1)) {
-			WAYPT_SET(this, speed, dist / labs(timed));
-			if (this->speed > tdata->max_spd) {
-				tdata->max_spd = this->speed;
+			if(!WAYPT_HAS(thisw, speed)) {
+				// Only recompute speed if the waypoint
+				// didn't already have a speed
+				WAYPT_SET(thisw, speed, dist / labs(timed));
 			}
-			if (this->speed < tdata->min_spd) {
-				tdata->min_spd = this->speed;
+			if (thisw->speed > tdata->max_spd) {
+				tdata->max_spd = thisw->speed;
+			}
+			if (thisw->speed < tdata->min_spd) {
+				tdata->min_spd = thisw->speed;
 			}
 		}
 
-		if ((this->altitude > 0) && (this->altitude < tdata->min_alt)) {
-			tdata->min_alt = this->altitude;
+		if ((thisw->altitude > 0) && (thisw->altitude < tdata->min_alt)) {
+			tdata->min_alt = thisw->altitude;
 		}
-		if (this->altitude > tdata->max_alt) {
-			tdata->max_alt = this->altitude;
+		if (thisw->altitude > tdata->max_alt) {
+			tdata->max_alt = thisw->altitude;
 		}
 
-		if (this->heartrate > 0) {
+		if (thisw->heartrate > 0) {
 			pts_hrt++;
-			tot_hrt += (float) this->heartrate;
+			tot_hrt += (float) thisw->heartrate;
 		}
 
-		if ((this->heartrate > 0) && (this->heartrate < tdata->min_hrt)) {
-			tdata->min_hrt = (int) this->heartrate;
+		if ((thisw->heartrate > 0) && (thisw->heartrate < tdata->min_hrt)) {
+			tdata->min_hrt = (int) thisw->heartrate;
 		}
 
-		if ((this->heartrate > 0) && (this->heartrate > tdata->max_hrt)) {
-			tdata->max_hrt = (int) this->heartrate;
+		if ((thisw->heartrate > 0) && (thisw->heartrate > tdata->max_hrt)) {
+			tdata->max_hrt = (int) thisw->heartrate;
 		}
 
-		if (this->cadence > 0) {
+		if (thisw->cadence > 0) {
 			pts_cad++;
-			tot_cad += (float) this->cadence;
+			tot_cad += (float) thisw->cadence;
 		}
 
-		if ((this->cadence > 0) && (this->cadence > tdata->max_cad)) {
-			tdata->max_cad = (int) this->cadence;
+		if ((thisw->cadence > 0) && (thisw->cadence > tdata->max_cad)) {
+			tdata->max_cad = (int) thisw->cadence;
 		}
 
-		if (this->creation_time && (this->creation_time < tdata->start)) {
-			tdata->start = this->creation_time;
+		if (thisw->creation_time && (thisw->creation_time < tdata->start)) {
+			tdata->start = thisw->creation_time;
 		}
 
-		if (this->creation_time > tdata->end) {
-			tdata->end = this->creation_time;
+		if (thisw->creation_time > tdata->end) {
+			tdata->end = thisw->creation_time;
 			if (tdata->start == 0) {
 				tdata->start = tdata->end;
 			}
 		}
-		prev = this;
-		if (!this->shortname || !this->shortname[0] ) {
+		prev = thisw;
+		if (!thisw->shortname || !thisw->shortname[0] ) {
 			snprintf(tkptname, sizeof(tkptname), "%s-%d", 
 				trk->rte_name ? trk->rte_name : "" , tkpt);
-			this->shortname = xstrdup(tkptname);
+			thisw->shortname = xstrdup(tkptname);
 		}
 		tkpt++;
 	}
