@@ -1,7 +1,7 @@
 /*
 
     Track manipulation filter
-
+    Copyright (c) 2009, 2010 Robert Lipe, robertlipe@gpsbabel.org
     Copyright (C) 2005-2006 Olaf Klein, o.b.klein@gpsbabel.org
 
     This program is free software; you can redistribute it and/or modify
@@ -32,6 +32,8 @@
     2006-06-01: Add name option
     2007-01-08: if not really needed disable check for valid timestamps
 		(based on patch from Vladimir Kondratiev)
+    2007-07-26: Allow 'range' together with trackpoints without timestamp
+    2010-06-02: Add specified timestamp to each trackpoint (added by sven_luzar)
  */
  
 #include <ctype.h>
@@ -39,8 +41,9 @@
 #include "filterdefs.h"
 #include "strptime.h"
 #include "grtcirc.h"
+#include "xmlgeneric.h"
 
-#if FILTERS_ENABLED
+#if FILTERS_ENABLED || MINIMAL_FILTERS
 #define MYNAME "trackfilter"
 
 #define TRACKFILTER_PACK_OPTION		"pack"
@@ -55,6 +58,10 @@
 #define TRACKFILTER_FIX_OPTION          "fix"
 #define TRACKFILTER_COURSE_OPTION       "course"
 #define TRACKFILTER_SPEED_OPTION        "speed"
+#define TRACKFILTER_SEG2TRK_OPTION      "seg2trk"
+#define TRACKFILTER_TRK2SEG_OPTION      "trk2seg"
+#define TRACKFILTER_SEGMENT_OPTION      "segment"
+#define TRACKFILTER_FAKETIME_OPTION     "faketime"
 
 #undef TRACKF_DBG
 
@@ -70,6 +77,10 @@ static char *opt_fix = NULL;
 static char *opt_course = NULL;
 static char *opt_speed = NULL;
 static char *opt_name = NULL;
+static char *opt_seg2trk = NULL;
+static char *opt_trk2seg = NULL;
+static char *opt_segment = NULL;
+static char *opt_faketime = NULL;
 
 static
 arglist_t trackfilter_args[] = {
@@ -105,6 +116,18 @@ arglist_t trackfilter_args[] = {
 	    NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
 	{TRACKFILTER_SPEED_OPTION, &opt_speed, "Synthesize speed",
             NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	{TRACKFILTER_SEG2TRK_OPTION, &opt_seg2trk,
+	    "Split track at segment boundaries into multiple tracks",
+            NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	{TRACKFILTER_TRK2SEG_OPTION, &opt_trk2seg,
+	    "Merge tracks inserting segment separators at boundaries",
+            NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	{TRACKFILTER_SEGMENT_OPTION, &opt_segment,
+	    "segment tracks with abnormally long gaps",
+	    NULL, ARGTYPE_BOOL, ARG_NOMINMAX },
+	{TRACKFILTER_FAKETIME_OPTION, &opt_faketime,
+	    "Add specified timestamp to each trackpoint",
+             NULL, ARGTYPE_STRING, ARG_NOMINMAX},
 	ARG_TERMINATOR
 };
 
@@ -181,8 +204,8 @@ trackfilter_parse_time_opt(const char *arg)
 static int
 trackfilter_init_qsort_cb(const void *a, const void *b)
 {
-	const trkflt_t *ra = a;
-	const trkflt_t *rb = b;
+	const trkflt_t *ra = (const trkflt_t*) a;
+	const trkflt_t *rb = (const trkflt_t*) b;
 
 	return ra->first_time - rb->first_time;
 }
@@ -197,24 +220,29 @@ trackfilter_merge_qsort_cb(const void *a, const void *b)
 }
 
 static fix_type
-trackfilter_parse_fix()
+trackfilter_parse_fix(int *nsats)
 {
 	if ( !opt_fix ) {
 		return fix_unknown;
 	}
 	if ( !case_ignore_strcmp( opt_fix, "pps" )) {
+		*nsats = 4;
 		return fix_pps;
 	}
 	if ( !case_ignore_strcmp( opt_fix, "dgps" )) {
+		*nsats = 4;
 		return fix_dgps;
 	}
 	if ( !case_ignore_strcmp( opt_fix, "3d" )) {
+		*nsats = 4;
 		return fix_3d;
 	}
 	if ( !case_ignore_strcmp( opt_fix, "2d" )) {
+		*nsats = 3;
 		return fix_2d;
 	}
 	if ( !case_ignore_strcmp( opt_fix, "none" )) {
+		*nsats = 0;
 		return fix_none;
 	}
 	fatal( MYNAME ": invalid fix type\n" );
@@ -239,6 +267,12 @@ trackfilter_fill_track_list_cb(const route_head *track) 	/* callback for track_d
 		if ((track->rte_name == NULL) ||
 		    (case_ignore_str_match(track->rte_name, opt_name) == 0))
 		{
+			QUEUE_FOR_EACH((queue *)&track->waypoint_list, elem, tmp)
+			{
+				waypoint *wpt = (waypoint *)elem;
+				track_del_wpt((route_head *)track, wpt);
+				waypt_free(wpt);
+			}
 			track_del_head((route_head *)track);
 			return;
 		}
@@ -254,8 +288,10 @@ trackfilter_fill_track_list_cb(const route_head *track) 	/* callback for track_d
 	    track_pts++;
 	    
 	    wpt = (waypoint *)elem;
-	    is_fatal((need_time != 0) && (wpt->creation_time == 0),
-		MYNAME "-init: Found track point without time!");
+	    if((need_time != 0) && (wpt->creation_time == 0)) {
+	      fatal(MYNAME "-init: Found track point at %f,%f without time!\n",
+                    wpt->latitude, wpt->longitude);
+            }
 
 	    i++;
 	    if (i == 1) 
@@ -266,8 +302,12 @@ trackfilter_fill_track_list_cb(const route_head *track) 	/* callback for track_d
 		
 	    if ((need_time != 0) && (prev != NULL) && (prev->creation_time > wpt->creation_time))
 	    {
-		if (opt_merge == NULL)
-		    fatal(MYNAME "-init: Track points badly ordered (timestamp)!\n");
+		if (opt_merge == NULL) {
+                    char t1[64], t2[64];
+                    xml_fill_in_time(t1, prev->creation_time, 0, XML_LONG_TIME);
+                    xml_fill_in_time(t2, wpt->creation_time, 0, XML_LONG_TIME);
+		    fatal(MYNAME "-init: Track points badly ordered (timestamp %s > %s)!\n", t1, t2);
+                    }
 	    }
 	    prev = wpt;
 	}
@@ -393,7 +433,8 @@ trackfilter_pack(void)
 	    QUEUE_FOR_EACH((queue *)&curr->waypoint_list, elem, tmp)
 	    {
 		waypoint *wpt = (waypoint *)elem;
-		route_add_wpt(master, waypt_dupe(wpt));
+		track_del_wpt(curr, wpt);
+		track_add_wpt(master, wpt);
 	    }
 	    track_del_head(curr);
 	    track_list[i].track = NULL;
@@ -417,7 +458,7 @@ trackfilter_merge(void)
 	
 	if (track_pts < 1) return;
 	
-	buff = xcalloc(track_pts, sizeof(*buff));
+	buff = (waypoint **)xcalloc(track_pts, sizeof(*buff));
 
 	j = 0;
 	for (i = 0; i < track_ct; i++)		/* put all points into temp buffer */
@@ -714,8 +755,9 @@ trackfilter_synth(void)
 	time_t oldtime = 0;
 	int first = 1;
 	fix_type fix;
+	int nsats = 0;
 	
-	fix = trackfilter_parse_fix();
+	fix = trackfilter_parse_fix(&nsats);
 	
 	for (i = 0; i < track_ct; i++)
 	{
@@ -726,32 +768,34 @@ trackfilter_synth(void)
 		wpt = (waypoint *)elem;
 		if ( opt_fix ) {
 			wpt->fix = fix;
+			if (wpt->sat == 0)
+				wpt->sat = nsats;
 		}
 		if ( first ) {
 			if ( opt_course ) {
-				wpt->course = 0;
+				WAYPT_SET(wpt, course, 0);
 			}
 			if ( opt_speed ) {
-				wpt->speed = 0;
+				WAYPT_SET(wpt, speed, 0);
 			}
 			first = 0;
 		}
 		else {
 			if ( opt_course ) {
-				wpt->course = heading_true_degrees( RAD(oldlat), 
+				WAYPT_SET(wpt, course, heading_true_degrees( RAD(oldlat), 
 					RAD(oldlon),RAD(wpt->latitude), 
-					RAD(wpt->longitude) );
+					RAD(wpt->longitude)) );
 			}
 			if ( opt_speed ) {
 				if ( oldtime != wpt->creation_time ) {
-					wpt->speed = radtometers(gcdist( 
+					WAYPT_SET(wpt, speed, radtometers(gcdist( 
 					    RAD(oldlat), RAD(oldlon), 
 					    RAD(wpt->latitude), 
 					    RAD(wpt->longitude))) /
-					    labs(wpt->creation_time-oldtime);
+					    labs(wpt->creation_time-oldtime));
 				}	
 				else {
-					wpt->speed = unknown_speed;
+					WAYPT_UNSET(wpt, speed);
 				}
 			}
 		}
@@ -799,7 +843,7 @@ trackfilter_range(void)		/* returns number of track points left after filtering
 {
 	time_t start, stop;
 	queue *elem, *tmp;
-	int i, dropped;
+	int i, dropped, inside = 0;
 	
 	if (opt_start != 0)
 	    start = trackfilter_range_check(opt_start);
@@ -811,7 +855,7 @@ trackfilter_range(void)		/* returns number of track points left after filtering
 	else
 	    stop = 0x7FFFFFFF;
 
-	dropped = 0;
+	dropped = inside = 0;
 	
 	for (i = 0; i < track_ct; i++)
 	{
@@ -820,9 +864,16 @@ trackfilter_range(void)		/* returns number of track points left after filtering
 	    QUEUE_FOR_EACH((queue *)&track->waypoint_list, elem, tmp)
 	    {
 		waypoint *wpt = (waypoint *)elem;
-
-		if ((wpt->creation_time < start) || (wpt->creation_time > stop))
-		{
+		if (wpt->creation_time > 0) {
+		    inside = ((wpt->creation_time >= start) && (wpt->creation_time <= stop));
+		}
+		// If the time is mangled so horribly that it's 
+		// negative, toss it.
+		if (wpt->creation_time < 0) {
+		    inside = 0;
+		}
+		
+		if (! inside) {
 		    track_del_wpt(track, wpt);
 		    waypt_free(wpt);
 		    dropped++;
@@ -842,6 +893,256 @@ trackfilter_range(void)		/* returns number of track points left after filtering
 	return track_pts - dropped;
 }
 
+/*******************************************************************************
+* option "seg2trk"
+*******************************************************************************/
+
+static void
+trackfilter_seg2trk(void)
+{
+	int i;
+
+	for (i = 0; i < track_ct; i++)
+	{
+	    queue *elem, *tmp;
+	    route_head *src = track_list[i].track;
+	    route_head *dest = NULL;
+	    route_head *insert_point = src;
+	    int trk_seg_num = 1, first = 1;
+
+	    QUEUE_FOR_EACH((queue *)&src->waypoint_list, elem, tmp)
+	    {
+		waypoint *wpt = (waypoint *)elem;
+		if (wpt->wpt_flags.new_trkseg && !first)
+		{
+		    char trk_seg_num_buf[10];
+
+		    dest = route_head_alloc();
+		    dest->rte_num = src->rte_num;
+		    /* name in the form TRACKNAME #n */
+		    snprintf(trk_seg_num_buf, sizeof(trk_seg_num_buf), "%d", ++trk_seg_num);
+		    dest->rte_name = xmalloc(strlen(src->rte_name)+strlen(trk_seg_num_buf)+3);
+		    sprintf(dest->rte_name, "%s #%s", src->rte_name, trk_seg_num_buf);
+
+		    /* Insert after original track or after last newly
+		     * created track */
+		    track_insert_head(dest, insert_point);
+		    insert_point = dest;
+		}
+
+		/* If we found a track separator, transfer from original to
+		 * new track. We have to reset new_trkseg temporarily to
+		 * prevent track_del_wpt() from copying it to the next track
+		 * point.
+		 */
+		if (dest)
+		{
+		    int orig_new_trkseg = wpt->wpt_flags.new_trkseg;
+		    wpt->wpt_flags.new_trkseg = 0;
+		    track_del_wpt(src, wpt);
+		    wpt->wpt_flags.new_trkseg = orig_new_trkseg;
+		    track_add_wpt(dest, wpt);
+		}
+		first = 0;
+	    }
+	}
+}
+
+/*******************************************************************************
+* option "trk2seg"
+*******************************************************************************/
+
+static void
+trackfilter_trk2seg(void)
+{
+	int i, first;
+	route_head *master;
+	
+	master = track_list[0].track;
+	
+	for (i = 1; i < track_ct; i++)
+	{
+	    queue *elem, *tmp;
+	    route_head *curr = track_list[i].track;
+	    
+	    first = 1;
+	    QUEUE_FOR_EACH((queue *)&curr->waypoint_list, elem, tmp)
+	    {
+		waypoint *wpt = (waypoint *)elem;
+
+
+		int orig_new_trkseg = wpt->wpt_flags.new_trkseg;
+		wpt->wpt_flags.new_trkseg = 0;
+		track_del_wpt(curr, wpt);
+		wpt->wpt_flags.new_trkseg = orig_new_trkseg;
+		track_add_wpt(master, wpt);
+		if (first)
+		{
+		    wpt->wpt_flags.new_trkseg = 1;
+		    first = 0;
+		}
+	    }
+	    track_del_head(curr);
+	    track_list[i].track = NULL;
+	}
+	track_ct = 1;
+}
+
+/*******************************************************************************
+* option: "faketime"
+*******************************************************************************/
+
+typedef struct faketime_s
+{
+       time_t start;
+       int    step;
+       int   force;
+} faketime_t;
+
+static faketime_t
+trackfilter_faketime_check(const char *timestr)
+{
+       int i, j;
+       char fmtstart[20];
+       char fmtstep[20];
+       char c;
+       const char *cin;
+       struct tm time;
+       int timeparse = 1;
+       faketime_t result;
+       result.force = 0;
+
+       i = j = 0;
+       strncpy(fmtstart, "00000101000000", sizeof(fmtstart));
+       strncpy(fmtstep,  "00000000000000", sizeof(fmtstep));
+       cin = timestr;
+
+       while ((c = *cin++))
+       {
+               if (c=='f') {
+                       result.force = 1;
+                       continue;
+               }
+
+               if (c!='+' && isdigit(c) == 0)
+                       fatal(MYNAME "-faketime: invalid character \"%c\"!\n", c);
+
+               if (timeparse) {
+                       if ((c == '+')) {
+                               fmtstart[i++] = '\0';
+                               timeparse = 0;
+                       } else {
+                               if (fmtstart[i] == '\0') fatal(MYNAME "-faketime: parameter too long \"%s\"!\n", timestr);
+                               fmtstart[i++] = c;
+                       }
+               } else {
+                       if (fmtstep[j] == '\0') fatal(MYNAME "-faketime: parameter too long \"%s\"!\n", timestr);
+                       fmtstep[j++] = c;
+               }
+       }
+       fmtstep[j++] = '\0';
+
+       cin = strptime(fmtstart, "%Y%m%d%H%M%S", &time);
+       result.step = atoi(fmtstep);
+       if ((cin != NULL) && (*cin != '\0'))
+           fatal(MYNAME "-faketime-check: Invalid time stamp (stopped at %s of %s)!\n", cin, fmtstart);
+
+       result.start = mkgmtime(&time);
+       return result;
+}
+
+static int
+trackfilter_faketime(void)             /* returns number of track points left after filtering */
+{
+       faketime_t faketime;
+
+       queue *elem, *tmp;
+       int i, dropped, inside = 0;
+
+       if (opt_faketime != 0)
+           faketime = trackfilter_faketime_check(opt_faketime);
+
+       dropped = inside = 0;
+
+       for (i = 0; i < track_ct; i++)
+       {
+           route_head *track = track_list[i].track;
+
+           QUEUE_FOR_EACH((queue *)&track->waypoint_list, elem, tmp)
+           {
+               waypoint *wpt = (waypoint *)elem;
+
+                       if (opt_faketime != 0 && (wpt->creation_time == 0 || faketime.force)) {
+                               wpt->creation_time = faketime.start;
+                               faketime.start += faketime.step;
+                       }
+           }
+       }
+
+       return track_pts - dropped;
+}
+
+static int
+trackfilter_points_are_same(const waypoint *wpta, const waypoint *wptb)
+{
+  // We use a simpler (non great circle) test for lat/lon here as this
+  // is used for keeping the 'bookends' of non-moving points.
+  return
+      abs(wpta->latitude - wptb->latitude) < .0000001 &&
+      abs(wpta->latitude - wptb->latitude) < .0000001 &&
+      abs(wpta->altitude - wptb->altitude) < 20 &&
+      (WAYPT_HAS(wpta,course) == WAYPT_HAS(wptb,course)) &&
+      (wpta->course == wptb->course) &&
+      (wpta->speed == wptb->speed) &&
+      (wpta->heartrate == wptb->heartrate) &&
+      (wpta->cadence == wptb->cadence) &&
+      (wpta->temperature == wptb->temperature);
+}
+
+static void
+trackfilter_segment_head(const route_head *rte)
+{
+  queue *elem, *tmp;
+  double avg_dist = 0;
+  int index = 0;
+  waypoint *prev_wpt = NULL;
+  // Consider tossing trackpoints closer than this in radians.
+  // (Empirically determined; It's a few dozen feet.)
+  const double ktoo_close = 0.000005;
+
+  QUEUE_FOR_EACH(&rte->waypoint_list, elem, tmp) {
+    waypoint *wpt = (waypoint *)elem;
+    if (index > 0) {
+      double cur_dist = gcdist(RAD(prev_wpt->latitude),
+                               RAD(prev_wpt->longitude),
+                               RAD(wpt->latitude),
+                               RAD(wpt->longitude));
+      // Denoise points that are on top of each other.
+      if (avg_dist == 0)
+        avg_dist = cur_dist;
+
+      if (cur_dist < ktoo_close) {
+        if (wpt != (waypoint *) QUEUE_LAST(&rte->waypoint_list)) {
+          waypoint *next_wpt = (waypoint *) QUEUE_NEXT(&wpt->Q);
+          if (trackfilter_points_are_same(prev_wpt, wpt) &&
+              trackfilter_points_are_same(wpt, next_wpt)) {
+            track_del_wpt((route_head *)rte, wpt);
+            continue;
+            }
+        }
+      }
+      if (cur_dist > .001 && cur_dist > 1.2* avg_dist) {
+        avg_dist = cur_dist = 0;
+        wpt->wpt_flags.new_trkseg = 1;
+      }
+      // Update weighted moving average;
+      avg_dist = (cur_dist + 4.0 * avg_dist) / 5.0;
+    }
+    prev_wpt = wpt;
+    index++;
+  }
+}
+
 /*******************************************************************************
 * global cb's
 *******************************************************************************/
@@ -861,7 +1162,7 @@ trackfilter_init(const char *args)
  */
 	need_time = ( 
 	    opt_merge || opt_pack || opt_split || opt_sdistance ||
-	    opt_move || opt_start || opt_stop || opt_fix || opt_speed ||
+	    opt_move || opt_fix || opt_speed ||
 	    (trackfilter_opt_count() == 0)	/* do pack by default */
 	);
 	/* in case of a formated title we also need valid timestamps */
@@ -869,6 +1170,11 @@ trackfilter_init(const char *args)
 	
 	track_ct = 0;
 	track_pts = 0;
+
+	// Perform segmenting first.
+	if (opt_segment) {
+		track_disp_all(trackfilter_segment_head, NULL, NULL);
+	}
 	
 	if (count > 0)
 	{
@@ -927,6 +1233,20 @@ trackfilter_process(void)
 	    if ( !opts ) return;
 	}
 
+       if ((opt_faketime != NULL))
+       {
+           opts--;
+
+           trackfilter_faketime();
+
+           if (opts == 0) return;
+
+           trackfilter_deinit();       /* reinitialize */
+           trackfilter_init(NULL);
+
+           if (track_ct == 0) return;          /* no more track(s), no more fun */
+       }
+
 	if ((opt_stop != NULL) || (opt_start != NULL))
 	{
 	    if (opt_start != NULL) opts--;
@@ -943,6 +1263,21 @@ trackfilter_process(void)
 	    
 	}
 	
+	if (opt_seg2trk != NULL)
+	{
+	    trackfilter_seg2trk();
+	    if (--opts == 0) return;
+	    
+	    trackfilter_deinit();	/* reinitialize */
+	    trackfilter_init(NULL);
+	}
+
+	if (opt_trk2seg != NULL)
+	{
+	    trackfilter_trk2seg();
+	    if (--opts == 0) return;
+	}
+
 	if (opt_title != NULL)
 	{
 	    if (--opts == 0)
