@@ -542,8 +542,8 @@ unicsv_fondle_header(char *ibuf)
 
 		if (column % 4 == 0) {
 			int sz = (column + 4) * sizeof(*unicsv_fields_tab);
-			if (column == 0) unicsv_fields_tab = xmalloc(sz);
-			else unicsv_fields_tab = xrealloc(unicsv_fields_tab, sz);
+			if (column == 0) unicsv_fields_tab = (field_e*) xmalloc(sz);
+			else unicsv_fields_tab = (field_e*) xrealloc(unicsv_fields_tab, sz);
 			for (i = 0; i < 4; i++) unicsv_fields_tab[column + i] = fld_terminator;
 		}
 
@@ -1192,7 +1192,7 @@ unicsv_print_data_time(const time_t atime)
 static void
 unicsv_waypt_enum_cb(const waypoint *wpt)
 {
-	char *shortname;
+	const char *shortname;
 	garmin_fs_t *gmsd;
 
 	shortname = (wpt->shortname) ? wpt->shortname : "";
@@ -1267,7 +1267,7 @@ unicsv_waypt_disp_cb(const waypoint *wpt)
 {
 	double lat, lon, alt;
 	char *cout = NULL;
-	char *shortname;
+	const char *shortname;
 	garmin_fs_t *gmsd;
 #ifdef UNICSV_GC_READY
 	const geocache_data *gc_data = NULL;
@@ -1372,7 +1372,7 @@ unicsv_waypt_disp_cb(const waypoint *wpt)
 		unicsv_print_str((wpt->icon_descr != NULL) ? wpt->icon_descr : "Waypoint");
 	if FIELD_USED(fld_depth) {
 		if WAYPT_HAS(wpt, depth)
-			gbfprintf(fout, "%s%.1f", unicsv_fieldsep, wpt->depth);
+			gbfprintf(fout, "%s%.3f", unicsv_fieldsep, wpt->depth);
 		else
 			gbfputs(unicsv_fieldsep, fout);
 	}
@@ -1384,13 +1384,13 @@ unicsv_waypt_disp_cb(const waypoint *wpt)
 	}
 	if FIELD_USED(fld_temperature) {
 		if WAYPT_HAS(wpt, temperature)
-			gbfprintf(fout, "%s%.1f", unicsv_fieldsep, wpt->temperature);
+			gbfprintf(fout, "%s%.3f", unicsv_fieldsep, wpt->temperature);
 		else
 			gbfputs(unicsv_fieldsep, fout);
 	}
 	if FIELD_USED(fld_speed) {
 		if WAYPT_HAS(wpt, speed)
-			gbfprintf(fout, "%s%.1f", unicsv_fieldsep, wpt->speed);
+			gbfprintf(fout, "%s%.2f", unicsv_fieldsep, wpt->speed);
 		else
 			gbfputs(unicsv_fieldsep, fout);
 	}
@@ -1415,19 +1415,19 @@ unicsv_waypt_disp_cb(const waypoint *wpt)
 	}
 	if FIELD_USED(fld_hdop) {
 		if (wpt->hdop > 0)
-			gbfprintf(fout, "%s%.1f", unicsv_fieldsep, wpt->hdop);
+			gbfprintf(fout, "%s%.2f", unicsv_fieldsep, wpt->hdop);
 		else
 			gbfputs(unicsv_fieldsep, fout);
 	}
 	if FIELD_USED(fld_vdop) {
 		if (wpt->vdop > 0)
-			gbfprintf(fout, "%s%.1f", unicsv_fieldsep, wpt->vdop);
+			gbfprintf(fout, "%s%.2f", unicsv_fieldsep, wpt->vdop);
 		else
 			gbfputs(unicsv_fieldsep, fout);
 	}
 	if FIELD_USED(fld_pdop) {
 		if (wpt->pdop > 0)
-			gbfprintf(fout, "%s%.1f", unicsv_fieldsep, wpt->pdop);
+			gbfprintf(fout, "%s%.2f", unicsv_fieldsep, wpt->pdop);
 		else
 			gbfputs(unicsv_fieldsep, fout);
 	}
